"""PubMed Tool for PraisonAI Agents.

Search biomedical literature on PubMed.

Usage:
    from praisonai_tools import PubMedTool
    
    pubmed = PubMedTool()
    results = pubmed.search("cancer treatment")
"""

import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class PubMedTool(BaseTool):
    """Tool for PubMed search."""
    
    name = "pubmed"
    description = "Search biomedical literature on PubMed."
    
    def __init__(self, email: Optional[str] = None):
        self.email = email
        super().__init__()
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        max_results: int = 10,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "search":
            return self.search(query=query, max_results=max_results)
        elif action == "get_article":
            return self.get_article(pmid=kwargs.get("pmid"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def search(self, query: str, max_results: int = 10) -> List[Dict[str, Any]]:
        """Search PubMed."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            from Bio import Entrez
        except ImportError:
            return [{"error": "biopython not installed. Install with: pip install biopython"}]
        
        try:
            Entrez.email = self.email or "user@example.com"
            handle = Entrez.esearch(db="pubmed", term=query, retmax=max_results)
            record = Entrez.read(handle)
            handle.close()
            
            ids = record.get("IdList", [])
            if not ids:
                return []
            
            handle = Entrez.efetch(db="pubmed", id=ids, rettype="abstract", retmode="xml")
            records = Entrez.read(handle)
            handle.close()
            
            results = []
            for article in records.get("PubmedArticle", []):
                medline = article.get("MedlineCitation", {})
                art = medline.get("Article", {})
                results.append({
                    "pmid": str(medline.get("PMID", "")),
                    "title": art.get("ArticleTitle", ""),
                    "abstract": art.get("Abstract", {}).get("AbstractText", [""])[0] if art.get("Abstract") else "",
                    "journal": art.get("Journal", {}).get("Title", ""),
                })
            return results
        except Exception as e:
            logger.error(f"PubMed search error: {e}")
            return [{"error": str(e)}]
    
    def get_article(self, pmid: str) -> Dict[str, Any]:
        """Get article by PMID."""
        if not pmid:
            return {"error": "pmid is required"}
        
        try:
            from Bio import Entrez
        except ImportError:
            return {"error": "biopython not installed"}
        
        try:
            Entrez.email = self.email or "user@example.com"
            handle = Entrez.efetch(db="pubmed", id=pmid, rettype="abstract", retmode="xml")
            records = Entrez.read(handle)
            handle.close()
            
            if not records.get("PubmedArticle"):
                return {"error": "Article not found"}
            
            article = records["PubmedArticle"][0]
            medline = article.get("MedlineCitation", {})
            art = medline.get("Article", {})
            
            return {
                "pmid": pmid,
                "title": art.get("ArticleTitle", ""),
                "abstract": art.get("Abstract", {}).get("AbstractText", [""])[0] if art.get("Abstract") else "",
                "journal": art.get("Journal", {}).get("Title", ""),
                "authors": [
                    f"{a.get('LastName', '')} {a.get('ForeName', '')}"
                    for a in art.get("AuthorList", [])
                ],
            }
        except Exception as e:
            logger.error(f"PubMed get_article error: {e}")
            return {"error": str(e)}


def pubmed_search(query: str, max_results: int = 10) -> List[Dict[str, Any]]:
    """Search PubMed."""
    return PubMedTool().search(query=query, max_results=max_results)



"""
Observability Configuration

Unified configuration for all observability providers.
"""

import os
from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional


@dataclass
class ObservabilityConfig:
    """Configuration for observability providers."""
    
    # Provider selection
    provider: Optional[str] = None  # Auto-detect if None
    
    # Common settings
    project_name: Optional[str] = None
    session_id: Optional[str] = None
    run_id: Optional[str] = None
    user_id: Optional[str] = None
    
    # Tracing settings
    enabled: bool = True
    trace_llm_calls: bool = True
    trace_tool_calls: bool = True
    trace_agent_steps: bool = True
    
    # Multi-agent settings
    propagate_context: bool = True
    parent_trace_id: Optional[str] = None
    
    # Performance settings
    batch_size: int = 100
    flush_interval: float = 5.0
    async_export: bool = True
    
    # Redaction settings
    redact_inputs: bool = False
    redact_outputs: bool = False
    redaction_patterns: List[str] = field(default_factory=list)
    
    # Provider-specific settings
    extra: Dict[str, Any] = field(default_factory=dict)
    
    @classmethod
    def from_env(cls) -> "ObservabilityConfig":
        """Create config from environment variables."""
        return cls(
            provider=os.getenv("PRAISONAI_OBS_PROVIDER"),
            project_name=os.getenv("PRAISONAI_OBS_PROJECT"),
            session_id=os.getenv("PRAISONAI_OBS_SESSION_ID"),
            run_id=os.getenv("PRAISONAI_OBS_RUN_ID"),
            user_id=os.getenv("PRAISONAI_OBS_USER_ID"),
            enabled=os.getenv("PRAISONAI_OBS_ENABLED", "true").lower() == "true",
            trace_llm_calls=os.getenv("PRAISONAI_OBS_TRACE_LLM", "true").lower() == "true",
            trace_tool_calls=os.getenv("PRAISONAI_OBS_TRACE_TOOLS", "true").lower() == "true",
            trace_agent_steps=os.getenv("PRAISONAI_OBS_TRACE_AGENTS", "true").lower() == "true",
            propagate_context=os.getenv("PRAISONAI_OBS_PROPAGATE", "true").lower() == "true",
            batch_size=int(os.getenv("PRAISONAI_OBS_BATCH_SIZE", "100")),
            flush_interval=float(os.getenv("PRAISONAI_OBS_FLUSH_INTERVAL", "5.0")),
            async_export=os.getenv("PRAISONAI_OBS_ASYNC", "true").lower() == "true",
            redact_inputs=os.getenv("PRAISONAI_OBS_REDACT_INPUTS", "false").lower() == "true",
            redact_outputs=os.getenv("PRAISONAI_OBS_REDACT_OUTPUTS", "false").lower() == "true",
        )
    
    def merge(self, **kwargs) -> "ObservabilityConfig":
        """Create a new config with merged values."""
        current = {
            "provider": self.provider,
            "project_name": self.project_name,
            "session_id": self.session_id,
            "run_id": self.run_id,
            "user_id": self.user_id,
            "enabled": self.enabled,
            "trace_llm_calls": self.trace_llm_calls,
            "trace_tool_calls": self.trace_tool_calls,
            "trace_agent_steps": self.trace_agent_steps,
            "propagate_context": self.propagate_context,
            "parent_trace_id": self.parent_trace_id,
            "batch_size": self.batch_size,
            "flush_interval": self.flush_interval,
            "async_export": self.async_export,
            "redact_inputs": self.redact_inputs,
            "redact_outputs": self.redact_outputs,
            "redaction_patterns": self.redaction_patterns.copy(),
            "extra": self.extra.copy(),
        }
        current.update({k: v for k, v in kwargs.items() if v is not None})
        return ObservabilityConfig(**current)


# Provider environment variable mappings
PROVIDER_ENV_KEYS = {
    "agentops": ["AGENTOPS_API_KEY"],
    "langfuse": ["LANGFUSE_PUBLIC_KEY", "LANGFUSE_SECRET_KEY"],
    "langsmith": ["LANGSMITH_API_KEY"],
    "traceloop": ["TRACELOOP_API_KEY"],
    "arize_phoenix": ["PHOENIX_API_KEY"],
    "openlit": ["OPENLIT_API_KEY"],
    "langtrace": ["LANGTRACE_API_KEY"],
    "langwatch": ["LANGWATCH_API_KEY"],
    "datadog": ["DD_API_KEY"],
    "mlflow": ["MLFLOW_TRACKING_URI"],
    "opik": ["OPIK_API_KEY", "COMET_API_KEY"],
    "portkey": ["PORTKEY_API_KEY"],
    "braintrust": ["BRAINTRUST_API_KEY"],
    "maxim": ["MAXIM_API_KEY"],
    "weave": ["WANDB_API_KEY"],
    "neatlogs": ["NEATLOGS_API_KEY"],
    "langdb": ["LANGDB_API_KEY"],
    "atla": ["ATLA_API_KEY"],
    "patronus": ["PATRONUS_API_KEY"],
    "truefoundry": ["TRUEFOUNDRY_API_KEY"],
}


def detect_provider() -> Optional[str]:
    """Auto-detect available provider from environment variables."""
    for provider, keys in PROVIDER_ENV_KEYS.items():
        if all(os.getenv(key) for key in keys):
            return provider
    return None



"""Jina Reader Tool for PraisonAI Agents.

Read and extract content from URLs using Jina Reader.

Usage:
    from praisonai_tools import JinaTool
    
    jina = JinaTool()
    content = jina.read("https://example.com")

Environment Variables:
    JINA_API_KEY: Jina API key (optional for basic usage)
"""

import os
import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class JinaTool(BaseTool):
    """Tool for reading web content using Jina Reader."""
    
    name = "jina"
    description = "Read and extract content from URLs using Jina Reader."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("JINA_API_KEY")
        super().__init__()
    
    def run(
        self,
        action: str = "read",
        url: Optional[str] = None,
        query: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        action = action.lower().replace("-", "_")
        
        if action == "read":
            return self.read(url=url)
        elif action == "search":
            return self.search(query=query)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def read(self, url: str) -> Dict[str, Any]:
        """Read content from URL."""
        if not url:
            return {"error": "url is required"}
        
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        try:
            headers = {}
            if self.api_key:
                headers["Authorization"] = f"Bearer {self.api_key}"
            
            response = requests.get(
                f"https://r.jina.ai/{url}",
                headers=headers,
                timeout=30,
            )
            
            if response.status_code != 200:
                return {"error": f"HTTP {response.status_code}"}
            
            return {
                "url": url,
                "content": response.text[:10000],
            }
        except Exception as e:
            logger.error(f"Jina read error: {e}")
            return {"error": str(e)}
    
    def search(self, query: str) -> Dict[str, Any]:
        """Search using Jina."""
        if not query:
            return {"error": "query is required"}
        
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        try:
            headers = {}
            if self.api_key:
                headers["Authorization"] = f"Bearer {self.api_key}"
            
            response = requests.get(
                f"https://s.jina.ai/{query}",
                headers=headers,
                timeout=30,
            )
            
            if response.status_code != 200:
                return {"error": f"HTTP {response.status_code}"}
            
            return {
                "query": query,
                "content": response.text[:10000],
            }
        except Exception as e:
            logger.error(f"Jina search error: {e}")
            return {"error": str(e)}


def jina_read(url: str) -> Dict[str, Any]:
    """Read URL with Jina."""
    return JinaTool().read(url=url)



"""Couchbase Tool for PraisonAI Agents.

Couchbase database operations.

Usage:
    from praisonai_tools import CouchbaseTool
    
    cb = CouchbaseTool()
    doc = cb.get(bucket="default", key="user:123")

Environment Variables:
    COUCHBASE_CONNECTION_STRING: Connection string
    COUCHBASE_USERNAME: Username
    COUCHBASE_PASSWORD: Password
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class CouchbaseTool(BaseTool):
    """Tool for Couchbase operations."""
    
    name = "couchbase"
    description = "Couchbase database operations."
    
    def __init__(
        self,
        connection_string: Optional[str] = None,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ):
        self.connection_string = connection_string or os.getenv("COUCHBASE_CONNECTION_STRING", "couchbase://localhost")
        self.username = username or os.getenv("COUCHBASE_USERNAME")
        self.password = password or os.getenv("COUCHBASE_PASSWORD")
        self._cluster = None
        super().__init__()
    
    @property
    def cluster(self):
        if self._cluster is None:
            try:
                from couchbase.cluster import Cluster
                from couchbase.options import ClusterOptions
                from couchbase.auth import PasswordAuthenticator
            except ImportError:
                raise ImportError("couchbase not installed")
            auth = PasswordAuthenticator(self.username, self.password)
            self._cluster = Cluster(self.connection_string, ClusterOptions(auth))
        return self._cluster
    
    def run(
        self,
        action: str = "get",
        bucket: Optional[str] = None,
        key: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "get":
            return self.get(bucket=bucket, key=key)
        elif action == "upsert":
            return self.upsert(bucket=bucket, key=key, value=kwargs.get("value"))
        elif action == "query":
            return self.query(sql=kwargs.get("sql"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def get(self, bucket: str, key: str) -> Dict[str, Any]:
        """Get document by key."""
        if not bucket or not key:
            return {"error": "bucket and key are required"}
        try:
            b = self.cluster.bucket(bucket)
            coll = b.default_collection()
            result = coll.get(key)
            return {"key": key, "value": result.content_as[dict]}
        except Exception as e:
            logger.error(f"Couchbase get error: {e}")
            return {"error": str(e)}
    
    def upsert(self, bucket: str, key: str, value: Dict) -> Dict[str, Any]:
        """Upsert document."""
        if not bucket or not key or not value:
            return {"error": "bucket, key, and value are required"}
        try:
            b = self.cluster.bucket(bucket)
            coll = b.default_collection()
            coll.upsert(key, value)
            return {"success": True}
        except Exception as e:
            logger.error(f"Couchbase upsert error: {e}")
            return {"error": str(e)}
    
    def query(self, sql: str) -> List[Dict[str, Any]]:
        """Execute N1QL query."""
        if not sql:
            return [{"error": "sql is required"}]
        try:
            result = self.cluster.query(sql)
            return [row for row in result]
        except Exception as e:
            logger.error(f"Couchbase query error: {e}")
            return [{"error": str(e)}]


def couchbase_get(bucket: str, key: str) -> Dict[str, Any]:
    """Get from Couchbase."""
    return CouchbaseTool().get(bucket=bucket, key=key)



"""Tests for LLM Tool."""

import os
import pytest
from unittest.mock import Mock, patch

import sys
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))

from llm_tool import LLMTool, LLMResponse, llm_complete


class TestLLMTool:
    """Test LLMTool class."""
    
    def test_init_default(self):
        """Test default initialization."""
        tool = LLMTool()
        assert tool.provider == "openai"
        assert tool.model == "gpt-4o-mini"
        assert tool.temperature == 0.7
        assert tool.max_tokens == 4096
    
    def test_init_custom(self):
        """Test custom initialization."""
        tool = LLMTool(
            provider="anthropic",
            model="claude-3-haiku",
            temperature=0.5,
            max_tokens=2048,
        )
        assert tool.provider == "anthropic"
        assert tool.model == "claude-3-haiku"
        assert tool.temperature == 0.5
        assert tool.max_tokens == 2048
    
    def test_default_model(self):
        """Test default model selection."""
        openai_tool = LLMTool(provider="openai")
        assert openai_tool.model == "gpt-4o-mini"
        
        anthropic_tool = LLMTool(provider="anthropic")
        assert anthropic_tool.model == "claude-3-haiku-20240307"
        
        google_tool = LLMTool(provider="google")
        assert google_tool.model == "gemini-1.5-flash"
    
    def test_check_dependencies_openai(self):
        """Test dependency check for OpenAI."""
        tool = LLMTool(provider="openai")
        deps = tool.check_dependencies()
        assert "openai" in deps
        assert "api_key" in deps
    
    @pytest.mark.skipif(not os.environ.get("OPENAI_API_KEY"), reason="OPENAI_API_KEY not set")
    def test_complete_openai(self):
        """Test completion with OpenAI."""
        tool = LLMTool(provider="openai")
        response = tool.complete("Say 'test' and nothing else", max_tokens=10)
        
        assert response.success
        assert response.content
        assert response.model
        assert response.provider == "openai"
        assert response.usage
    
    @pytest.mark.skipif(not os.environ.get("OPENAI_API_KEY"), reason="OPENAI_API_KEY not set")
    def test_complete_with_system(self):
        """Test completion with system message."""
        tool = LLMTool(provider="openai")
        response = tool.complete(
            "What are you?",
            system="You are a helpful robot. Always respond with 'I am a robot.'",
            max_tokens=20,
        )
        
        assert response.success
        assert "robot" in response.content.lower()
    
    @pytest.mark.skipif(not os.environ.get("OPENAI_API_KEY"), reason="OPENAI_API_KEY not set")
    def test_extract_json(self):
        """Test JSON extraction."""
        tool = LLMTool(provider="openai")
        schema = {"name": "string", "age": "number"}
        
        result = tool.extract_json(
            "Extract: John is 30 years old",
            schema=schema,
        )
        
        assert isinstance(result, dict)
        assert "name" in result or "age" in result


class TestLLMResponse:
    """Test LLMResponse dataclass."""
    
    def test_response_creation(self):
        """Test response creation."""
        response = LLMResponse(
            success=True,
            content="Hello",
            model="gpt-4o-mini",
            provider="openai",
            usage={"total_tokens": 10},
            finish_reason="stop",
        )
        
        assert response.success
        assert response.content == "Hello"
        assert response.model == "gpt-4o-mini"
        assert response.provider == "openai"
        assert response.usage["total_tokens"] == 10


class TestConvenienceFunctions:
    """Test convenience functions."""
    
    @pytest.mark.skipif(not os.environ.get("OPENAI_API_KEY"), reason="OPENAI_API_KEY not set")
    def test_llm_complete(self):
        """Test llm_complete function."""
        result = llm_complete("Say 'hello'", max_tokens=10)
        assert isinstance(result, str)
        assert len(result) > 0



"""E2B Tool for PraisonAI Agents.

Run code in E2B sandboxes.

Usage:
    from praisonai_tools import E2BTool
    
    e2b = E2BTool()
    result = e2b.run_code("print('Hello')", language="python")

Environment Variables:
    E2B_API_KEY: E2B API key
"""

import os
import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class E2BTool(BaseTool):
    """Tool for E2B code execution."""
    
    name = "e2b"
    description = "Run code in E2B sandboxes."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("E2B_API_KEY")
        super().__init__()
    
    def run(
        self,
        action: str = "run_code",
        code: Optional[str] = None,
        language: str = "python",
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        if action == "run_code":
            return self.run_code(code=code, language=language)
        return {"error": f"Unknown action: {action}"}
    
    def run_code(self, code: str, language: str = "python") -> Dict[str, Any]:
        """Run code in sandbox."""
        if not code:
            return {"error": "code is required"}
        if not self.api_key:
            return {"error": "E2B_API_KEY required"}
        
        try:
            from e2b_code_interpreter import CodeInterpreter
        except ImportError:
            return {"error": "e2b-code-interpreter not installed. Install with: pip install e2b-code-interpreter"}
        
        try:
            with CodeInterpreter() as sandbox:
                execution = sandbox.notebook.exec_cell(code)
                return {
                    "stdout": execution.logs.stdout,
                    "stderr": execution.logs.stderr,
                    "results": [str(r) for r in execution.results],
                }
        except Exception as e:
            logger.error(f"E2B run_code error: {e}")
            return {"error": str(e)}


def e2b_run_code(code: str, language: str = "python") -> Dict[str, Any]:
    """Run code in E2B."""
    return E2BTool().run_code(code=code, language=language)



"""Daytona Tool for PraisonAI Agents.

Manage Daytona development environments.

Usage:
    from praisonai_tools import DaytonaTool
    
    daytona = DaytonaTool()
    workspaces = daytona.list_workspaces()

Environment Variables:
    DAYTONA_API_KEY: Daytona API key
    DAYTONA_SERVER_URL: Daytona server URL
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class DaytonaTool(BaseTool):
    """Tool for Daytona dev environments."""
    
    name = "daytona"
    description = "Manage Daytona development environments."
    
    def __init__(
        self,
        api_key: Optional[str] = None,
        server_url: Optional[str] = None,
    ):
        self.api_key = api_key or os.getenv("DAYTONA_API_KEY")
        self.server_url = server_url or os.getenv("DAYTONA_SERVER_URL", "https://api.daytona.io")
        super().__init__()
    
    def _request(self, method: str, endpoint: str, data: Dict = None) -> Dict:
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        if not self.api_key:
            return {"error": "DAYTONA_API_KEY required"}
        
        url = f"{self.server_url}/{endpoint}"
        headers = {"Authorization": f"Bearer {self.api_key}"}
        
        try:
            if method == "GET":
                resp = requests.get(url, headers=headers, timeout=10)
            elif method == "POST":
                resp = requests.post(url, headers=headers, json=data, timeout=30)
            elif method == "DELETE":
                resp = requests.delete(url, headers=headers, timeout=10)
                if resp.status_code == 204:
                    return {"success": True}
            else:
                return {"error": f"Unknown method: {method}"}
            return resp.json()
        except Exception as e:
            return {"error": str(e)}
    
    def run(
        self,
        action: str = "list_workspaces",
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "list_workspaces":
            return self.list_workspaces()
        elif action == "create_workspace":
            return self.create_workspace(**kwargs)
        elif action == "delete_workspace":
            return self.delete_workspace(workspace_id=kwargs.get("workspace_id"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def list_workspaces(self) -> List[Dict[str, Any]]:
        """List workspaces."""
        result = self._request("GET", "workspaces")
        if isinstance(result, dict) and "error" in result:
            return [result]
        return result if isinstance(result, list) else [result]
    
    def create_workspace(self, name: str, repository: str) -> Dict[str, Any]:
        """Create workspace."""
        if not name or not repository:
            return {"error": "name and repository are required"}
        return self._request("POST", "workspaces", {"name": name, "repository": repository})
    
    def delete_workspace(self, workspace_id: str) -> Dict[str, Any]:
        """Delete workspace."""
        if not workspace_id:
            return {"error": "workspace_id is required"}
        return self._request("DELETE", f"workspaces/{workspace_id}")


def daytona_list_workspaces() -> List[Dict[str, Any]]:
    """List Daytona workspaces."""
    return DaytonaTool().list_workspaces()



"""OpenCV Tool for PraisonAI Agents.

Image processing using OpenCV.

Usage:
    from praisonai_tools import OpenCVTool
    
    cv = OpenCVTool()
    result = cv.resize("image.jpg", width=800)
"""

import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class OpenCVTool(BaseTool):
    """Tool for OpenCV image processing."""
    
    name = "opencv"
    description = "Image processing using OpenCV."
    
    def run(
        self,
        action: str = "resize",
        image_path: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        action = action.lower().replace("-", "_")
        
        if action == "resize":
            return self.resize(image_path=image_path, **kwargs)
        elif action == "grayscale":
            return self.grayscale(image_path=image_path, **kwargs)
        elif action == "blur":
            return self.blur(image_path=image_path, **kwargs)
        elif action == "detect_faces":
            return self.detect_faces(image_path=image_path)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def resize(self, image_path: str, width: int = None, height: int = None, output_path: str = None) -> Dict[str, Any]:
        """Resize image."""
        if not image_path:
            return {"error": "image_path is required"}
        
        try:
            import cv2
        except ImportError:
            return {"error": "opencv-python not installed"}
        
        try:
            img = cv2.imread(image_path)
            if img is None:
                return {"error": "Could not read image"}
            
            h, w = img.shape[:2]
            if width and not height:
                height = int(h * width / w)
            elif height and not width:
                width = int(w * height / h)
            elif not width and not height:
                return {"error": "width or height required"}
            
            resized = cv2.resize(img, (width, height))
            out = output_path or image_path.replace(".", "_resized.")
            cv2.imwrite(out, resized)
            return {"success": True, "output_path": out, "width": width, "height": height}
        except Exception as e:
            logger.error(f"OpenCV resize error: {e}")
            return {"error": str(e)}
    
    def grayscale(self, image_path: str, output_path: str = None) -> Dict[str, Any]:
        """Convert to grayscale."""
        if not image_path:
            return {"error": "image_path is required"}
        
        try:
            import cv2
        except ImportError:
            return {"error": "opencv-python not installed"}
        
        try:
            img = cv2.imread(image_path)
            gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
            out = output_path or image_path.replace(".", "_gray.")
            cv2.imwrite(out, gray)
            return {"success": True, "output_path": out}
        except Exception as e:
            logger.error(f"OpenCV grayscale error: {e}")
            return {"error": str(e)}
    
    def blur(self, image_path: str, kernel_size: int = 5, output_path: str = None) -> Dict[str, Any]:
        """Apply blur."""
        if not image_path:
            return {"error": "image_path is required"}
        
        try:
            import cv2
        except ImportError:
            return {"error": "opencv-python not installed"}
        
        try:
            img = cv2.imread(image_path)
            blurred = cv2.GaussianBlur(img, (kernel_size, kernel_size), 0)
            out = output_path or image_path.replace(".", "_blur.")
            cv2.imwrite(out, blurred)
            return {"success": True, "output_path": out}
        except Exception as e:
            logger.error(f"OpenCV blur error: {e}")
            return {"error": str(e)}
    
    def detect_faces(self, image_path: str) -> Dict[str, Any]:
        """Detect faces in image."""
        if not image_path:
            return {"error": "image_path is required"}
        
        try:
            import cv2
        except ImportError:
            return {"error": "opencv-python not installed"}
        
        try:
            img = cv2.imread(image_path)
            gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
            face_cascade = cv2.CascadeClassifier(cv2.data.haarcascades + "haarcascade_frontalface_default.xml")
            faces = face_cascade.detectMultiScale(gray, 1.1, 4)
            return {"faces_count": len(faces), "faces": [{"x": int(x), "y": int(y), "w": int(w), "h": int(h)} for x, y, w, h in faces]}
        except Exception as e:
            logger.error(f"OpenCV detect_faces error: {e}")
            return {"error": str(e)}


def opencv_resize(image_path: str, width: int) -> Dict[str, Any]:
    """Resize image."""
    return OpenCVTool().resize(image_path=image_path, width=width)



"""Unit tests for EditIntent schema."""

import pytest
from praisonai_tools.fcp_tool.intent import (
    EditIntent,
    Asset,
    Segment,
    Timeline,
    Project,
    ProjectFormat,
    Operations,
    seconds_to_rational,
    rational_to_seconds,
    get_format_preset,
    FORMAT_PRESETS,
)


class TestProjectFormat:
    """Tests for ProjectFormat model."""

    def test_defaults(self):
        """Test default values."""
        fmt = ProjectFormat()
        assert fmt.width == 1920
        assert fmt.height == 1080
        assert fmt.fps == 25.0
        assert fmt.frame_duration_rational == "100/2500s"

    def test_custom_fps(self):
        """Test custom fps calculates correct frame duration."""
        fmt = ProjectFormat(fps=30.0)
        assert fmt.frame_duration_rational == "100/3000s"

    def test_get_timescale(self):
        """Test timescale calculation."""
        fmt = ProjectFormat(fps=25.0)
        assert fmt.get_timescale() == 2500

        fmt30 = ProjectFormat(fps=30.0)
        assert fmt30.get_timescale() == 3000


class TestAsset:
    """Tests for Asset model."""

    def test_uid_generation(self):
        """Test UID is generated from path."""
        asset = Asset(id="r2", name="test", path="/path/to/file.mov")
        assert asset.uid is not None
        assert len(asset.uid) == 32

    def test_same_path_same_uid(self):
        """Test same path generates same UID."""
        asset1 = Asset(id="r2", name="test1", path="/path/to/file.mov")
        asset2 = Asset(id="r3", name="test2", path="/path/to/file.mov")
        assert asset1.uid == asset2.uid

    def test_different_path_different_uid(self):
        """Test different paths generate different UIDs."""
        asset1 = Asset(id="r2", name="test1", path="/path/to/file1.mov")
        asset2 = Asset(id="r3", name="test2", path="/path/to/file2.mov")
        assert asset1.uid != asset2.uid

    def test_src_url(self):
        """Test file URL generation."""
        asset = Asset(id="r2", name="test", path="/path/to/file.mov")
        assert asset.get_src_url() == "file:///path/to/file.mov"

    def test_path_must_be_absolute(self):
        """Test that relative paths are rejected."""
        with pytest.raises(ValueError, match="must be absolute"):
            Asset(id="r2", name="test", path="relative/path.mov")


class TestSegment:
    """Tests for Segment model."""

    def test_rational_time_validation(self):
        """Test rational time format validation."""
        segment = Segment(
            asset_id="r2",
            offset="0/2500s",
            start="0/2500s",
            duration="5000/2500s",
        )
        assert segment.offset == "0/2500s"

    def test_invalid_rational_time(self):
        """Test invalid rational time is rejected."""
        with pytest.raises(ValueError, match="Invalid rational time"):
            Segment(
                asset_id="r2",
                offset="invalid",
                start="0/2500s",
                duration="5000/2500s",
            )


class TestOperations:
    """Tests for Operations model."""

    def test_unimplemented_warnings(self):
        """Test warnings for unimplemented operations."""
        ops = Operations(loudness_target_lufs=-16.0)
        warnings = ops.get_unimplemented_warnings()
        assert len(warnings) == 1
        assert "loudness" in warnings[0].lower()

    def test_multiple_warnings(self):
        """Test multiple unimplemented operations."""
        ops = Operations(
            loudness_target_lufs=-16.0,
            remove_pauses_over_seconds=2.0,
        )
        warnings = ops.get_unimplemented_warnings()
        assert len(warnings) == 2


class TestEditIntent:
    """Tests for EditIntent model."""

    def test_minimal_intent(self):
        """Test creating minimal valid intent."""
        intent = EditIntent(
            project=Project(name="Test"),
            assets=[Asset(id="r2", name="clip", path="/tmp/test.mov")],
            timeline=Timeline(segments=[
                Segment(asset_id="r2", offset="0/2500s", start="0/2500s", duration="5000/2500s")
            ])
        )
        assert intent.version == "1"
        assert len(intent.assets) == 1
        assert len(intent.timeline.segments) == 1

    def test_asset_reference_validation(self):
        """Test that invalid asset references are rejected."""
        with pytest.raises(ValueError, match="unknown asset_id"):
            EditIntent(
                project=Project(name="Test"),
                assets=[Asset(id="r2", name="clip", path="/tmp/test.mov")],
                timeline=Timeline(segments=[
                    Segment(asset_id="r99", offset="0/2500s", start="0/2500s", duration="5000/2500s")
                ])
            )

    def test_get_asset_by_id(self):
        """Test getting asset by ID."""
        intent = EditIntent(
            project=Project(name="Test"),
            assets=[
                Asset(id="r2", name="clip1", path="/tmp/test1.mov"),
                Asset(id="r3", name="clip2", path="/tmp/test2.mov"),
            ],
            timeline=Timeline(segments=[])
        )
        asset = intent.get_asset_by_id("r3")
        assert asset is not None
        assert asset.name == "clip2"

        assert intent.get_asset_by_id("r99") is None

    def test_get_warnings(self):
        """Test collecting warnings."""
        intent = EditIntent(
            project=Project(name="Test"),
            assets=[Asset(id="r2", name="clip", path="/tmp/test.mov")],
            timeline=Timeline(segments=[]),
            operations=Operations(loudness_target_lufs=-16.0),
            needs_user_timestamps=True,
        )
        warnings = intent.get_warnings()
        assert len(warnings) >= 2


class TestTimeConversion:
    """Tests for time conversion utilities."""

    def test_seconds_to_rational(self):
        """Test seconds to rational conversion."""
        assert seconds_to_rational(1.0, fps=25.0) == "2500/2500s"
        assert seconds_to_rational(2.0, fps=25.0) == "5000/2500s"
        assert seconds_to_rational(0.5, fps=25.0) == "1250/2500s"

    def test_rational_to_seconds(self):
        """Test rational to seconds conversion."""
        assert rational_to_seconds("2500/2500s") == 1.0
        assert rational_to_seconds("5000/2500s") == 2.0
        assert rational_to_seconds("1250/2500s") == 0.5

    def test_roundtrip(self):
        """Test conversion roundtrip."""
        original = 3.5
        rational = seconds_to_rational(original, fps=25.0)
        result = rational_to_seconds(rational)
        assert abs(result - original) < 0.001


class TestFormatPresets:
    """Tests for format presets."""

    def test_preset_exists(self):
        """Test getting existing preset."""
        preset = get_format_preset("1080p25")
        assert preset.width == 1920
        assert preset.height == 1080
        assert preset.fps == 25.0

    def test_preset_not_found(self):
        """Test error for unknown preset."""
        with pytest.raises(ValueError, match="Unknown format preset"):
            get_format_preset("invalid_preset")

    def test_all_presets_valid(self):
        """Test all presets are valid."""
        for name in FORMAT_PRESETS:
            preset = get_format_preset(name)
            assert preset.width > 0
            assert preset.height > 0
            assert preset.fps > 0



"""
Arize Phoenix Provider

Integration with Arize Phoenix for LLM observability.
https://phoenix.arize.com/
"""

import os
from typing import Optional

from praisonai_tools.observability.base import (
    BaseObservabilityProvider,
    Span,
    Trace,
)
from praisonai_tools.observability.config import ObservabilityConfig
from praisonai_tools.observability.manager import ObservabilityManager


class ArizePhoenixProvider(BaseObservabilityProvider):
    """Arize Phoenix observability provider."""
    
    name = "arize_phoenix"
    
    def __init__(self, config: Optional[ObservabilityConfig] = None):
        """Initialize Arize Phoenix provider."""
        super().__init__(config)
        self._tracer_provider = None
    
    def is_available(self) -> bool:
        """Check if phoenix SDK is available."""
        try:
            import importlib.util
            return importlib.util.find_spec("phoenix.otel") is not None
        except ImportError:
            return False
    
    def init(self, **kwargs) -> bool:
        """Initialize Arize Phoenix."""
        if not self.is_available():
            return False
        
        try:
            from phoenix.otel import register
            
            api_key = kwargs.get("api_key") or os.getenv("PHOENIX_API_KEY")
            endpoint = kwargs.get("endpoint") or os.getenv(
                "PHOENIX_COLLECTOR_ENDPOINT", "https://app.phoenix.arize.com/"
            )
            project_name = kwargs.get("project_name") or self.config.project_name or "default"
            
            # Set environment variables
            if api_key:
                os.environ["PHOENIX_API_KEY"] = api_key
            os.environ["PHOENIX_COLLECTOR_ENDPOINT"] = endpoint
            
            # Register Phoenix tracer
            self._tracer_provider = register(
                project_name=project_name,
                auto_instrument=kwargs.get("auto_instrument", True),
            )
            
            self._initialized = True
            return True
            
        except Exception as e:
            print(f"Arize Phoenix init failed: {e}")
            return False
    
    def shutdown(self) -> None:
        """Shutdown Arize Phoenix provider."""
        if self._tracer_provider:
            try:
                self._tracer_provider.shutdown()
            except Exception:
                pass
        self._initialized = False
    
    def check_connection(self) -> tuple:
        """Check Arize Phoenix connection."""
        if not self._initialized:
            return False, "Arize Phoenix not initialized"
        
        api_key = os.getenv("PHOENIX_API_KEY")
        if api_key:
            return True, "Phoenix API key configured"
        return False, "PHOENIX_API_KEY not set"
    
    def export_span(self, span: Span) -> bool:
        """Export span to Arize Phoenix."""
        if not self._initialized:
            return False
        # Phoenix auto-instruments via OpenInference
        return True
    
    def export_trace(self, trace_obj: Trace) -> bool:
        """Export trace to Arize Phoenix."""
        if not self._initialized:
            return False
        return True


# Auto-register provider
ObservabilityManager.register_provider("arize_phoenix", ArizePhoenixProvider)



"""
Portkey Provider

Integration with Portkey for LLM gateway and observability.
https://portkey.ai/
"""

import os
from typing import Optional

from praisonai_tools.observability.base import (
    BaseObservabilityProvider,
    Span,
    Trace,
)
from praisonai_tools.observability.config import ObservabilityConfig
from praisonai_tools.observability.manager import ObservabilityManager


class PortkeyProvider(BaseObservabilityProvider):
    """Portkey observability provider."""
    
    name = "portkey"
    
    def __init__(self, config: Optional[ObservabilityConfig] = None):
        """Initialize Portkey provider."""
        super().__init__(config)
        self._portkey = None
        self._gateway_url = None
    
    def is_available(self) -> bool:
        """Check if portkey SDK is available."""
        try:
            import importlib.util
            return importlib.util.find_spec("portkey_ai") is not None
        except ImportError:
            return False
    
    def init(self, **kwargs) -> bool:
        """Initialize Portkey."""
        if not self.is_available():
            return False
        
        try:
            from portkey_ai import PORTKEY_GATEWAY_URL
            
            api_key = kwargs.get("api_key") or os.getenv("PORTKEY_API_KEY")
            
            if not api_key:
                return False
            
            self._gateway_url = PORTKEY_GATEWAY_URL
            self._initialized = True
            return True
            
        except Exception as e:
            print(f"Portkey init failed: {e}")
            return False
    
    def shutdown(self) -> None:
        """Shutdown Portkey provider."""
        self._initialized = False
    
    def check_connection(self) -> tuple:
        """Check Portkey connection."""
        if not self._initialized:
            return False, "Portkey not initialized"
        
        api_key = os.getenv("PORTKEY_API_KEY")
        if api_key:
            return True, "Portkey API key configured"
        return False, "PORTKEY_API_KEY not set"
    
    def get_headers(self, virtual_key: str = None, trace_id: str = None):
        """Get Portkey headers for LLM requests."""
        if not self.is_available():
            return {}
        
        try:
            from portkey_ai import createHeaders
            
            api_key = os.getenv("PORTKEY_API_KEY")
            return createHeaders(
                api_key=api_key,
                virtual_key=virtual_key,
                trace_id=trace_id,
            )
        except Exception:
            return {}
    
    def export_span(self, span: Span) -> bool:
        """Export span to Portkey."""
        if not self._initialized:
            return False
        return True
    
    def export_trace(self, trace_obj: Trace) -> bool:
        """Export trace to Portkey."""
        if not self._initialized:
            return False
        return True


ObservabilityManager.register_provider("portkey", PortkeyProvider)



"""Gmail Tool for PraisonAI Agents.

Send and read emails via Gmail API.

Usage:
    from praisonai_tools import GmailTool
    
    gmail = GmailTool()
    emails = gmail.list_emails()

Environment Variables:
    GMAIL_CREDENTIALS_FILE: Path to credentials.json
    GMAIL_TOKEN_FILE: Path to token.json
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class GmailTool(BaseTool):
    """Tool for Gmail operations."""
    
    name = "gmail"
    description = "Send and read emails via Gmail API."
    
    def __init__(
        self,
        credentials_file: Optional[str] = None,
        token_file: Optional[str] = None,
    ):
        self.credentials_file = credentials_file or os.getenv("GMAIL_CREDENTIALS_FILE", "credentials.json")
        self.token_file = token_file or os.getenv("GMAIL_TOKEN_FILE", "token.json")
        self._service = None
        super().__init__()
    
    @property
    def service(self):
        if self._service is None:
            try:
                from google.oauth2.credentials import Credentials
                from google_auth_oauthlib.flow import InstalledAppFlow
                from google.auth.transport.requests import Request
                from googleapiclient.discovery import build
            except ImportError:
                raise ImportError("google-api-python-client, google-auth-oauthlib required")
            
            SCOPES = ["https://www.googleapis.com/auth/gmail.modify"]
            creds = None
            
            if os.path.exists(self.token_file):
                creds = Credentials.from_authorized_user_file(self.token_file, SCOPES)
            
            if not creds or not creds.valid:
                if creds and creds.expired and creds.refresh_token:
                    creds.refresh(Request())
                else:
                    flow = InstalledAppFlow.from_client_secrets_file(self.credentials_file, SCOPES)
                    creds = flow.run_local_server(port=0)
                with open(self.token_file, "w") as token:
                    token.write(creds.to_json())
            
            self._service = build("gmail", "v1", credentials=creds)
        return self._service
    
    def run(
        self,
        action: str = "list",
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "list":
            return self.list_emails(**kwargs)
        elif action == "get":
            return self.get_email(message_id=kwargs.get("message_id"))
        elif action == "send":
            return self.send_email(**kwargs)
        elif action == "search":
            return self.search_emails(query=kwargs.get("query"), **kwargs)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def list_emails(self, max_results: int = 10, label: str = "INBOX") -> List[Dict[str, Any]]:
        """List emails."""
        try:
            results = self.service.users().messages().list(
                userId="me", labelIds=[label], maxResults=max_results
            ).execute()
            
            messages = results.get("messages", [])
            emails = []
            for msg in messages:
                email_data = self.service.users().messages().get(
                    userId="me", id=msg["id"], format="metadata",
                    metadataHeaders=["From", "Subject", "Date"]
                ).execute()
                
                headers = {h["name"]: h["value"] for h in email_data.get("payload", {}).get("headers", [])}
                emails.append({
                    "id": msg["id"],
                    "from": headers.get("From", ""),
                    "subject": headers.get("Subject", ""),
                    "date": headers.get("Date", ""),
                    "snippet": email_data.get("snippet", ""),
                })
            return emails
        except Exception as e:
            logger.error(f"Gmail list error: {e}")
            return [{"error": str(e)}]
    
    def get_email(self, message_id: str) -> Dict[str, Any]:
        """Get email by ID."""
        if not message_id:
            return {"error": "message_id is required"}
        
        try:
            email_data = self.service.users().messages().get(
                userId="me", id=message_id, format="full"
            ).execute()
            
            headers = {h["name"]: h["value"] for h in email_data.get("payload", {}).get("headers", [])}
            
            body = ""
            payload = email_data.get("payload", {})
            if "body" in payload and payload["body"].get("data"):
                import base64
                body = base64.urlsafe_b64decode(payload["body"]["data"]).decode("utf-8")
            elif "parts" in payload:
                for part in payload["parts"]:
                    if part.get("mimeType") == "text/plain" and part.get("body", {}).get("data"):
                        import base64
                        body = base64.urlsafe_b64decode(part["body"]["data"]).decode("utf-8")
                        break
            
            return {
                "id": message_id,
                "from": headers.get("From", ""),
                "to": headers.get("To", ""),
                "subject": headers.get("Subject", ""),
                "date": headers.get("Date", ""),
                "body": body,
            }
        except Exception as e:
            logger.error(f"Gmail get error: {e}")
            return {"error": str(e)}
    
    def send_email(self, to: str, subject: str, body: str) -> Dict[str, Any]:
        """Send email."""
        if not to or not subject:
            return {"error": "to and subject are required"}
        
        try:
            import base64
            from email.mime.text import MIMEText
            
            message = MIMEText(body)
            message["to"] = to
            message["subject"] = subject
            raw = base64.urlsafe_b64encode(message.as_bytes()).decode("utf-8")
            
            result = self.service.users().messages().send(
                userId="me", body={"raw": raw}
            ).execute()
            
            return {"success": True, "id": result.get("id")}
        except Exception as e:
            logger.error(f"Gmail send error: {e}")
            return {"error": str(e)}
    
    def search_emails(self, query: str, max_results: int = 10) -> List[Dict[str, Any]]:
        """Search emails."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            results = self.service.users().messages().list(
                userId="me", q=query, maxResults=max_results
            ).execute()
            
            messages = results.get("messages", [])
            emails = []
            for msg in messages:
                email_data = self.service.users().messages().get(
                    userId="me", id=msg["id"], format="metadata",
                    metadataHeaders=["From", "Subject", "Date"]
                ).execute()
                
                headers = {h["name"]: h["value"] for h in email_data.get("payload", {}).get("headers", [])}
                emails.append({
                    "id": msg["id"],
                    "from": headers.get("From", ""),
                    "subject": headers.get("Subject", ""),
                    "date": headers.get("Date", ""),
                })
            return emails
        except Exception as e:
            logger.error(f"Gmail search error: {e}")
            return [{"error": str(e)}]


def list_gmail_emails(max_results: int = 10) -> List[Dict[str, Any]]:
    """List Gmail emails."""
    return GmailTool().list_emails(max_results=max_results)



"""Zendesk Tool for PraisonAI Agents.

Manage Zendesk tickets and users.

Usage:
    from praisonai_tools import ZendeskTool
    
    zendesk = ZendeskTool()
    tickets = zendesk.list_tickets()

Environment Variables:
    ZENDESK_SUBDOMAIN: Zendesk subdomain
    ZENDESK_EMAIL: Zendesk user email
    ZENDESK_API_TOKEN: Zendesk API token
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class ZendeskTool(BaseTool):
    """Tool for Zendesk support operations."""
    
    name = "zendesk"
    description = "Manage Zendesk tickets and users."
    
    def __init__(
        self,
        subdomain: Optional[str] = None,
        email: Optional[str] = None,
        api_token: Optional[str] = None,
    ):
        self.subdomain = subdomain or os.getenv("ZENDESK_SUBDOMAIN")
        self.email = email or os.getenv("ZENDESK_EMAIL")
        self.api_token = api_token or os.getenv("ZENDESK_API_TOKEN")
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                from zenpy import Zenpy
            except ImportError:
                raise ImportError("zenpy not installed. Install with: pip install zenpy")
            
            if not all([self.subdomain, self.email, self.api_token]):
                raise ValueError("ZENDESK_SUBDOMAIN, ZENDESK_EMAIL, and ZENDESK_API_TOKEN required")
            
            self._client = Zenpy(
                subdomain=self.subdomain,
                email=self.email,
                token=self.api_token,
            )
        return self._client
    
    def run(
        self,
        action: str = "list_tickets",
        ticket_id: Optional[int] = None,
        subject: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "list_tickets":
            return self.list_tickets(**kwargs)
        elif action == "get_ticket":
            return self.get_ticket(ticket_id=ticket_id)
        elif action == "create_ticket":
            return self.create_ticket(subject=subject, **kwargs)
        elif action == "update_ticket":
            return self.update_ticket(ticket_id=ticket_id, **kwargs)
        elif action == "search_tickets":
            return self.search_tickets(query=kwargs.get("query"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def list_tickets(self, limit: int = 20) -> List[Dict[str, Any]]:
        """List tickets."""
        try:
            tickets = []
            for ticket in self.client.tickets(limit=limit):
                tickets.append({
                    "id": ticket.id,
                    "subject": ticket.subject,
                    "status": ticket.status,
                    "priority": ticket.priority,
                    "created_at": str(ticket.created_at),
                })
                if len(tickets) >= limit:
                    break
            return tickets
        except Exception as e:
            logger.error(f"Zendesk list_tickets error: {e}")
            return [{"error": str(e)}]
    
    def get_ticket(self, ticket_id: int) -> Dict[str, Any]:
        """Get ticket details."""
        if not ticket_id:
            return {"error": "ticket_id is required"}
        
        try:
            ticket = self.client.tickets(id=ticket_id)
            return {
                "id": ticket.id,
                "subject": ticket.subject,
                "description": ticket.description,
                "status": ticket.status,
                "priority": ticket.priority,
                "requester_id": ticket.requester_id,
                "created_at": str(ticket.created_at),
                "updated_at": str(ticket.updated_at),
            }
        except Exception as e:
            logger.error(f"Zendesk get_ticket error: {e}")
            return {"error": str(e)}
    
    def create_ticket(self, subject: str, description: str = None, priority: str = "normal") -> Dict[str, Any]:
        """Create a ticket."""
        if not subject:
            return {"error": "subject is required"}
        
        try:
            from zenpy.lib.api_objects import Ticket
            
            ticket = Ticket(
                subject=subject,
                description=description or subject,
                priority=priority,
            )
            result = self.client.tickets.create(ticket)
            return {
                "success": True,
                "id": result.ticket.id,
                "subject": result.ticket.subject,
            }
        except Exception as e:
            logger.error(f"Zendesk create_ticket error: {e}")
            return {"error": str(e)}
    
    def update_ticket(self, ticket_id: int, status: str = None, priority: str = None) -> Dict[str, Any]:
        """Update a ticket."""
        if not ticket_id:
            return {"error": "ticket_id is required"}
        
        try:
            ticket = self.client.tickets(id=ticket_id)
            if status:
                ticket.status = status
            if priority:
                ticket.priority = priority
            self.client.tickets.update(ticket)
            return {"success": True, "id": ticket_id}
        except Exception as e:
            logger.error(f"Zendesk update_ticket error: {e}")
            return {"error": str(e)}
    
    def search_tickets(self, query: str) -> List[Dict[str, Any]]:
        """Search tickets."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            tickets = []
            for ticket in self.client.search(query, type="ticket"):
                tickets.append({
                    "id": ticket.id,
                    "subject": ticket.subject,
                    "status": ticket.status,
                })
            return tickets[:20]
        except Exception as e:
            logger.error(f"Zendesk search_tickets error: {e}")
            return [{"error": str(e)}]


def list_zendesk_tickets(limit: int = 20) -> List[Dict[str, Any]]:
    """List Zendesk tickets."""
    return ZendeskTool().list_tickets(limit=limit)



"""Oxylabs Tool for PraisonAI Agents.

Web scraping using Oxylabs.

Usage:
    from praisonai_tools import OxylabsTool
    
    oxy = OxylabsTool()
    content = oxy.scrape("https://example.com")

Environment Variables:
    OXYLABS_USERNAME: Oxylabs username
    OXYLABS_PASSWORD: Oxylabs password
"""

import os
import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class OxylabsTool(BaseTool):
    """Tool for Oxylabs web scraping."""
    
    name = "oxylabs"
    description = "Web scraping using Oxylabs."
    
    def __init__(
        self,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ):
        self.username = username or os.getenv("OXYLABS_USERNAME")
        self.password = password or os.getenv("OXYLABS_PASSWORD")
        super().__init__()
    
    def run(
        self,
        action: str = "scrape",
        url: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        if action == "scrape":
            return self.scrape(url=url)
        return {"error": f"Unknown action: {action}"}
    
    def scrape(self, url: str) -> Dict[str, Any]:
        """Scrape URL."""
        if not url:
            return {"error": "url is required"}
        if not self.username or not self.password:
            return {"error": "OXYLABS_USERNAME and OXYLABS_PASSWORD required"}
        
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        try:
            payload = {"source": "universal", "url": url}
            resp = requests.post(
                "https://realtime.oxylabs.io/v1/queries",
                auth=(self.username, self.password),
                json=payload,
                timeout=60,
            )
            data = resp.json()
            results = data.get("results", [{}])
            return {
                "url": url,
                "content": results[0].get("content", "") if results else "",
            }
        except Exception as e:
            logger.error(f"Oxylabs scrape error: {e}")
            return {"error": str(e)}


def oxylabs_scrape(url: str) -> Dict[str, Any]:
    """Scrape with Oxylabs."""
    return OxylabsTool().scrape(url=url)



"""Fal Tool for PraisonAI Agents.

Run AI models on Fal.ai.

Usage:
    from praisonai_tools import FalTool
    
    fal = FalTool()
    output = fal.run_model("fal-ai/flux/schnell", {"prompt": "a cat"})

Environment Variables:
    FAL_KEY: Fal API key
"""

import os
import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class FalTool(BaseTool):
    """Tool for Fal.ai models."""
    
    name = "fal"
    description = "Run AI models on Fal.ai."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("FAL_KEY")
        super().__init__()
    
    def run(
        self,
        action: str = "run_model",
        model: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        if action == "run_model":
            return self.run_model(model=model, input_data=kwargs.get("input_data", kwargs))
        return {"error": f"Unknown action: {action}"}
    
    def run_model(self, model: str, input_data: Dict) -> Dict[str, Any]:
        """Run a model."""
        if not model:
            return {"error": "model is required"}
        if not self.api_key:
            return {"error": "FAL_KEY required"}
        
        try:
            import fal_client
        except ImportError:
            return {"error": "fal-client not installed. Install with: pip install fal-client"}
        
        try:
            result = fal_client.subscribe(model, arguments=input_data)
            return {"output": result}
        except Exception as e:
            logger.error(f"Fal run_model error: {e}")
            return {"error": str(e)}


def fal_run(model: str, input_data: Dict) -> Dict[str, Any]:
    """Run model on Fal."""
    return FalTool().run_model(model=model, input_data=input_data)



"""Integration test: praisonai-tools custom tools with praisonaiagents built-in tools.

This test verifies:
1. Built-in tools from praisonaiagents work (tavily_search)
2. Custom tools from praisonai-tools work (@tool decorator)
3. Both can be combined in an Agent

Usage:
    export OPENAI_API_KEY=your_openai_key
    export TAVILY_API_KEY=your_tavily_key
    python test_integration.py
"""

import os
import sys

print("=" * 60)
print("PraisonAI Tools Integration Test")
print("=" * 60)

# Test 1: Verify praisonai-tools imports work
print("\n[Test 1] Importing praisonai-tools...")
try:
    from praisonai_tools import BaseTool, tool, FunctionTool, is_tool, get_tool_schema
    print("✅ praisonai-tools imports successful")
except ImportError as e:
    print(f"❌ Failed to import praisonai-tools: {e}")
    sys.exit(1)

# Test 2: Verify praisonaiagents imports work
print("\n[Test 2] Importing praisonaiagents...")
try:
    from praisonaiagents import Agent
    from praisonaiagents.tools import tavily_search
    print("✅ praisonaiagents imports successful")
except ImportError as e:
    print(f"❌ Failed to import praisonaiagents: {e}")
    sys.exit(1)

# Test 3: Create custom tool with @tool decorator
print("\n[Test 3] Creating custom tool with @tool decorator...")

@tool
def my_calculator(expression: str) -> str:
    """Calculate a mathematical expression.
    
    Args:
        expression: Math expression to evaluate (e.g., '2 + 2')
    """
    try:
        result = eval(expression)
        return f"Result: {result}"
    except Exception as e:
        return f"Error: {e}"

print(f"✅ Custom tool created: {my_calculator.name}")
print(f"   Description: {my_calculator.description}")
print(f"   Is tool: {is_tool(my_calculator)}")

# Test 4: Get schema from custom tool
print("\n[Test 4] Getting OpenAI schema from custom tool...")
schema = get_tool_schema(my_calculator)
print(f"✅ Schema generated:")
print(f"   Function name: {schema['function']['name']}")
print(f"   Parameters: {list(schema['function']['parameters']['properties'].keys())}")

# Test 5: Execute custom tool directly
print("\n[Test 5] Executing custom tool directly...")
result = my_calculator(expression="10 * 5 + 2")
print(f"✅ Tool execution result: {result}")

# Test 6: Verify built-in tavily_search tool
print("\n[Test 6] Checking built-in tavily_search tool...")
print(f"✅ tavily_search function: {tavily_search.__name__}")

# Test 7: Test tavily_search with real API call (if key available)
tavily_key = os.environ.get("TAVILY_API_KEY")
if tavily_key:
    print("\n[Test 7] Testing tavily_search with real API call...")
    try:
        result = tavily_search("Python programming", max_results=2)
        if "error" not in result:
            print(f"✅ Tavily search successful!")
            print(f"   Found {len(result.get('results', []))} results")
            for r in result.get('results', [])[:2]:
                print(f"   - {r.get('title', 'No title')[:50]}...")
        else:
            print(f"⚠️ Tavily returned error: {result.get('error')}")
    except Exception as e:
        print(f"❌ Tavily search failed: {e}")
else:
    print("\n[Test 7] Skipped - TAVILY_API_KEY not set")

# Test 8: Create Agent with combined tools (without calling LLM)
print("\n[Test 8] Creating Agent with combined tools...")
openai_key = os.environ.get("OPENAI_API_KEY")
if openai_key:
    try:
        agent = Agent(
            instructions="You are a helpful assistant",
            tools=[tavily_search, my_calculator],
            verbose=False
        )
        print(f"✅ Agent created with {len(agent.tools)} tools:")
        for t in agent.tools:
            name = getattr(t, '__name__', getattr(t, 'name', str(t)))
            print(f"   - {name}")
    except Exception as e:
        print(f"❌ Agent creation failed: {e}")
else:
    print("⚠️ Skipped Agent creation - OPENAI_API_KEY not set")
    print("   To test full integration, set OPENAI_API_KEY")

print("\n" + "=" * 60)
print("Integration test completed!")
print("=" * 60)



"""
Langtrace Provider

Integration with Langtrace for LLM observability.
https://langtrace.ai/
"""

import os
from typing import Optional

from praisonai_tools.observability.base import (
    BaseObservabilityProvider,
    Span,
    Trace,
)
from praisonai_tools.observability.config import ObservabilityConfig
from praisonai_tools.observability.manager import ObservabilityManager


class LangtraceProvider(BaseObservabilityProvider):
    """Langtrace observability provider."""
    
    name = "langtrace"
    
    def __init__(self, config: Optional[ObservabilityConfig] = None):
        """Initialize Langtrace provider."""
        super().__init__(config)
        self._langtrace = None
    
    def is_available(self) -> bool:
        """Check if langtrace SDK is available."""
        try:
            import importlib.util
            return importlib.util.find_spec("langtrace_python_sdk") is not None
        except ImportError:
            return False
    
    def init(self, **kwargs) -> bool:
        """Initialize Langtrace."""
        if not self.is_available():
            return False
        
        try:
            from langtrace_python_sdk import langtrace
            
            api_key = kwargs.get("api_key") or os.getenv("LANGTRACE_API_KEY")
            
            if api_key:
                os.environ["LANGTRACE_API_KEY"] = api_key
            
            langtrace.init()
            
            self._langtrace = langtrace
            self._initialized = True
            return True
            
        except Exception as e:
            print(f"Langtrace init failed: {e}")
            return False
    
    def shutdown(self) -> None:
        """Shutdown Langtrace provider."""
        self._initialized = False
    
    def check_connection(self) -> tuple:
        """Check Langtrace connection."""
        if not self._initialized:
            return False, "Langtrace not initialized"
        
        api_key = os.getenv("LANGTRACE_API_KEY")
        if api_key:
            return True, "Langtrace API key configured"
        return False, "LANGTRACE_API_KEY not set"
    
    def export_span(self, span: Span) -> bool:
        """Export span to Langtrace."""
        if not self._initialized:
            return False
        return True
    
    def export_trace(self, trace_obj: Trace) -> bool:
        """Export trace to Langtrace."""
        if not self._initialized:
            return False
        return True


# Auto-register provider
ObservabilityManager.register_provider("langtrace", LangtraceProvider)



"""Knowledge Tool for PraisonAI Agents.

Knowledge base operations with document storage and retrieval.

Usage:
    from praisonai_tools import KnowledgeTool
    
    kb = KnowledgeTool()
    kb.add_document("doc1", "This is some content about AI")
"""

import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class KnowledgeTool(BaseTool):
    """Tool for knowledge base operations."""
    
    name = "knowledge"
    description = "Knowledge base operations with document storage and retrieval."
    
    def __init__(self):
        self._documents = {}
        super().__init__()
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "add":
            return self.add_document(doc_id=kwargs.get("doc_id"), content=kwargs.get("content"), metadata=kwargs.get("metadata"))
        elif action == "search":
            return self.search(query=query, **kwargs)
        elif action == "get":
            return self.get_document(doc_id=kwargs.get("doc_id"))
        elif action == "delete":
            return self.delete_document(doc_id=kwargs.get("doc_id"))
        elif action == "list":
            return self.list_documents()
        else:
            return {"error": f"Unknown action: {action}"}
    
    def add_document(self, doc_id: str, content: str, metadata: Dict = None) -> Dict[str, Any]:
        """Add document to knowledge base."""
        if not doc_id or not content:
            return {"error": "doc_id and content are required"}
        
        self._documents[doc_id] = {
            "content": content,
            "metadata": metadata or {},
        }
        return {"success": True, "doc_id": doc_id}
    
    def search(self, query: str, limit: int = 10) -> List[Dict[str, Any]]:
        """Search documents (simple text matching)."""
        if not query:
            return [{"error": "query is required"}]
        
        query_lower = query.lower()
        results = []
        
        for doc_id, doc in self._documents.items():
            content = doc["content"].lower()
            if query_lower in content:
                score = content.count(query_lower) / len(content.split())
                results.append({
                    "doc_id": doc_id,
                    "content": doc["content"][:500],
                    "metadata": doc["metadata"],
                    "score": score,
                })
        
        results.sort(key=lambda x: x["score"], reverse=True)
        return results[:limit]
    
    def get_document(self, doc_id: str) -> Dict[str, Any]:
        """Get document by ID."""
        if not doc_id:
            return {"error": "doc_id is required"}
        
        if doc_id not in self._documents:
            return {"error": "Document not found"}
        
        doc = self._documents[doc_id]
        return {"doc_id": doc_id, "content": doc["content"], "metadata": doc["metadata"]}
    
    def delete_document(self, doc_id: str) -> Dict[str, Any]:
        """Delete document."""
        if not doc_id:
            return {"error": "doc_id is required"}
        
        if doc_id not in self._documents:
            return {"error": "Document not found"}
        
        del self._documents[doc_id]
        return {"success": True}
    
    def list_documents(self) -> List[Dict[str, Any]]:
        """List all documents."""
        return [
            {"doc_id": doc_id, "metadata": doc["metadata"], "content_length": len(doc["content"])}
            for doc_id, doc in self._documents.items()
        ]


def knowledge_search(query: str, limit: int = 10) -> List[Dict[str, Any]]:
    """Search knowledge base."""
    return KnowledgeTool().search(query=query, limit=limit)



"""Brandfetch Tool for PraisonAI Agents.

Get brand assets and information.

Usage:
    from praisonai_tools import BrandfetchTool
    
    bf = BrandfetchTool()
    brand = bf.get_brand("google.com")

Environment Variables:
    BRANDFETCH_API_KEY: Brandfetch API key
"""

import os
import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class BrandfetchTool(BaseTool):
    """Tool for Brandfetch brand data."""
    
    name = "brandfetch"
    description = "Get brand assets and information."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("BRANDFETCH_API_KEY")
        super().__init__()
    
    def run(
        self,
        action: str = "get_brand",
        domain: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        if action == "get_brand":
            return self.get_brand(domain=domain)
        return {"error": f"Unknown action: {action}"}
    
    def get_brand(self, domain: str) -> Dict[str, Any]:
        """Get brand information."""
        if not domain:
            return {"error": "domain is required"}
        if not self.api_key:
            return {"error": "BRANDFETCH_API_KEY required"}
        
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        try:
            headers = {"Authorization": f"Bearer {self.api_key}"}
            resp = requests.get(f"https://api.brandfetch.io/v2/brands/{domain}", headers=headers, timeout=10)
            return resp.json()
        except Exception as e:
            logger.error(f"Brandfetch error: {e}")
            return {"error": str(e)}


def brandfetch_get_brand(domain: str) -> Dict[str, Any]:
    """Get brand info."""
    return BrandfetchTool().get_brand(domain=domain)



"""Valyu Tool for PraisonAI Agents.

Search using Valyu API.

Usage:
    from praisonai_tools import ValyuTool
    
    valyu = ValyuTool()
    results = valyu.search("AI news")

Environment Variables:
    VALYU_API_KEY: Valyu API key
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class ValyuTool(BaseTool):
    """Tool for Valyu search."""
    
    name = "valyu"
    description = "Search using Valyu API."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("VALYU_API_KEY")
        super().__init__()
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        if action == "search":
            return self.search(query=query, **kwargs)
        return {"error": f"Unknown action: {action}"}
    
    def search(self, query: str, max_results: int = 10) -> List[Dict[str, Any]]:
        """Search Valyu."""
        if not query:
            return [{"error": "query is required"}]
        if not self.api_key:
            return [{"error": "VALYU_API_KEY required"}]
        
        try:
            import requests
        except ImportError:
            return [{"error": "requests not installed"}]
        
        try:
            headers = {"Authorization": f"Bearer {self.api_key}"}
            data = {"query": query, "max_results": max_results}
            resp = requests.post(
                "https://api.valyu.network/v1/search",
                headers=headers,
                json=data,
                timeout=30,
            )
            result = resp.json()
            return result.get("results", [])
        except Exception as e:
            logger.error(f"Valyu search error: {e}")
            return [{"error": str(e)}]


def valyu_search(query: str, max_results: int = 10) -> List[Dict[str, Any]]:
    """Search with Valyu."""
    return ValyuTool().search(query=query, max_results=max_results)



"""Bitbucket Tool for PraisonAI Agents.

Manage Bitbucket repositories and pull requests.

Usage:
    from praisonai_tools import BitbucketTool
    
    bb = BitbucketTool()
    repos = bb.list_repos(workspace="myworkspace")

Environment Variables:
    BITBUCKET_USERNAME: Bitbucket username
    BITBUCKET_APP_PASSWORD: Bitbucket app password
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class BitbucketTool(BaseTool):
    """Tool for Bitbucket operations."""
    
    name = "bitbucket"
    description = "Manage Bitbucket repositories and pull requests."
    
    def __init__(
        self,
        username: Optional[str] = None,
        app_password: Optional[str] = None,
    ):
        self.username = username or os.getenv("BITBUCKET_USERNAME")
        self.app_password = app_password or os.getenv("BITBUCKET_APP_PASSWORD")
        super().__init__()
    
    def _request(self, method: str, endpoint: str, data: Dict = None) -> Dict:
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed"}
        
        if not self.username or not self.app_password:
            return {"error": "BITBUCKET_USERNAME and BITBUCKET_APP_PASSWORD required"}
        
        url = f"https://api.bitbucket.org/2.0/{endpoint}"
        auth = (self.username, self.app_password)
        
        try:
            if method == "GET":
                resp = requests.get(url, auth=auth, timeout=10)
            elif method == "POST":
                resp = requests.post(url, auth=auth, json=data, timeout=10)
            else:
                return {"error": f"Unknown method: {method}"}
            return resp.json()
        except Exception as e:
            return {"error": str(e)}
    
    def run(
        self,
        action: str = "list_repos",
        workspace: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "list_repos":
            return self.list_repos(workspace=workspace)
        elif action == "get_repo":
            return self.get_repo(workspace=workspace, repo_slug=kwargs.get("repo_slug"))
        elif action == "list_prs":
            return self.list_prs(workspace=workspace, repo_slug=kwargs.get("repo_slug"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def list_repos(self, workspace: str) -> List[Dict[str, Any]]:
        """List repositories."""
        if not workspace:
            return [{"error": "workspace is required"}]
        result = self._request("GET", f"repositories/{workspace}")
        if "error" in result:
            return [result]
        return [{"name": r["name"], "slug": r["slug"]} for r in result.get("values", [])]
    
    def get_repo(self, workspace: str, repo_slug: str) -> Dict[str, Any]:
        """Get repository details."""
        if not workspace or not repo_slug:
            return {"error": "workspace and repo_slug are required"}
        return self._request("GET", f"repositories/{workspace}/{repo_slug}")
    
    def list_prs(self, workspace: str, repo_slug: str) -> List[Dict[str, Any]]:
        """List pull requests."""
        if not workspace or not repo_slug:
            return [{"error": "workspace and repo_slug are required"}]
        result = self._request("GET", f"repositories/{workspace}/{repo_slug}/pullrequests")
        if "error" in result:
            return [result]
        return [{"id": pr["id"], "title": pr["title"], "state": pr["state"]} for pr in result.get("values", [])]


def bitbucket_list_repos(workspace: str) -> List[Dict[str, Any]]:
    """List Bitbucket repos."""
    return BitbucketTool().list_repos(workspace=workspace)



"""WebBrowser Tool for PraisonAI Agents.

Browser automation using Playwright.

Usage:
    from praisonai_tools import WebBrowserTool
    
    browser = WebBrowserTool()
    content = browser.get_page("https://example.com")
"""

import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class WebBrowserTool(BaseTool):
    """Tool for web browser automation."""
    
    name = "webbrowser"
    description = "Browser automation using Playwright."
    
    def __init__(self, headless: bool = True):
        self.headless = headless
        super().__init__()
    
    def run(
        self,
        action: str = "get_page",
        url: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        action = action.lower().replace("-", "_")
        
        if action == "get_page":
            return self.get_page(url=url)
        elif action == "screenshot":
            return self.screenshot(url=url, output_path=kwargs.get("output_path"))
        elif action == "click":
            return self.click(url=url, selector=kwargs.get("selector"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def get_page(self, url: str) -> Dict[str, Any]:
        """Get page content."""
        if not url:
            return {"error": "url is required"}
        
        try:
            from playwright.sync_api import sync_playwright
        except ImportError:
            return {"error": "playwright not installed. Install with: pip install playwright && playwright install"}
        
        try:
            with sync_playwright() as p:
                browser = p.chromium.launch(headless=self.headless)
                page = browser.new_page()
                page.goto(url)
                content = page.content()
                title = page.title()
                browser.close()
                return {"url": url, "title": title, "content": content[:10000]}
        except Exception as e:
            logger.error(f"WebBrowser get_page error: {e}")
            return {"error": str(e)}
    
    def screenshot(self, url: str, output_path: str = "screenshot.png") -> Dict[str, Any]:
        """Take screenshot."""
        if not url:
            return {"error": "url is required"}
        
        try:
            from playwright.sync_api import sync_playwright
        except ImportError:
            return {"error": "playwright not installed"}
        
        try:
            with sync_playwright() as p:
                browser = p.chromium.launch(headless=self.headless)
                page = browser.new_page()
                page.goto(url)
                page.screenshot(path=output_path)
                browser.close()
                return {"success": True, "output_path": output_path}
        except Exception as e:
            logger.error(f"WebBrowser screenshot error: {e}")
            return {"error": str(e)}
    
    def click(self, url: str, selector: str) -> Dict[str, Any]:
        """Click element on page."""
        if not url or not selector:
            return {"error": "url and selector are required"}
        
        try:
            from playwright.sync_api import sync_playwright
        except ImportError:
            return {"error": "playwright not installed"}
        
        try:
            with sync_playwright() as p:
                browser = p.chromium.launch(headless=self.headless)
                page = browser.new_page()
                page.goto(url)
                page.click(selector)
                content = page.content()
                browser.close()
                return {"success": True, "content": content[:10000]}
        except Exception as e:
            logger.error(f"WebBrowser click error: {e}")
            return {"error": str(e)}


def webbrowser_get_page(url: str) -> Dict[str, Any]:
    """Get page content."""
    return WebBrowserTool().get_page(url=url)



"""
Chart Tool - Generate charts and visualizations from data.

Supports various chart types including bar, line, pie, scatter, etc.
Uses matplotlib for rendering with optional AI-assisted chart selection.
"""

import os
import logging
from dataclasses import dataclass, field
from typing import Optional, List, Dict, Any, Union
from pathlib import Path

try:
    from .base import RecipeToolBase, RecipeToolResult
except ImportError:
    from base import RecipeToolBase, RecipeToolResult

logger = logging.getLogger(__name__)


@dataclass
class ChartResult(RecipeToolResult):
    """Result from chart generation."""
    output_path: str = ""
    chart_type: str = ""
    title: str = ""
    data_points: int = 0


class ChartTool(RecipeToolBase):
    """Chart generation tool using matplotlib."""
    
    CHART_TYPES = ["bar", "line", "pie", "scatter", "histogram", "area", "heatmap"]
    
    def __init__(
        self,
        style: str = "seaborn-v0_8-whitegrid",
        figsize: tuple = (10, 6),
        dpi: int = 150,
    ):
        self.style = style
        self.figsize = figsize
        self.dpi = dpi
    
    def check_dependencies(self) -> Dict[str, bool]:
        """Check if required dependencies are available."""
        deps = {}
        try:
            import matplotlib
            deps["matplotlib"] = True
        except ImportError:
            deps["matplotlib"] = False
        
        try:
            import pandas
            deps["pandas"] = True
        except ImportError:
            deps["pandas"] = False
        
        return deps
    
    def _setup_style(self):
        """Setup matplotlib style."""
        import matplotlib.pyplot as plt
        try:
            plt.style.use(self.style)
        except OSError:
            plt.style.use("ggplot")
    
    def bar(
        self,
        data: Dict[str, Union[int, float]],
        output_path: str,
        title: str = "Bar Chart",
        xlabel: str = "",
        ylabel: str = "",
        color: str = "#3498db",
        horizontal: bool = False,
    ) -> ChartResult:
        """
        Create a bar chart.
        
        Args:
            data: Dictionary of labels to values
            output_path: Path to save chart
            title: Chart title
            xlabel: X-axis label
            ylabel: Y-axis label
            color: Bar color
            horizontal: If True, create horizontal bar chart
        """
        import matplotlib.pyplot as plt
        
        self._setup_style()
        fig, ax = plt.subplots(figsize=self.figsize)
        
        labels = list(data.keys())
        values = list(data.values())
        
        if horizontal:
            ax.barh(labels, values, color=color)
        else:
            ax.bar(labels, values, color=color)
        
        ax.set_title(title)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        
        plt.tight_layout()
        plt.savefig(output_path, dpi=self.dpi, bbox_inches='tight')
        plt.close()
        
        return ChartResult(
            success=True,
            output_path=output_path,
            chart_type="bar",
            title=title,
            data_points=len(data),
        )
    
    def line(
        self,
        data: Union[Dict[str, List], List[Dict]],
        output_path: str,
        title: str = "Line Chart",
        xlabel: str = "",
        ylabel: str = "",
        legend: bool = True,
    ) -> ChartResult:
        """
        Create a line chart.
        
        Args:
            data: Dict with 'x' and 'y' keys, or list of series dicts
            output_path: Path to save chart
            title: Chart title
            xlabel: X-axis label
            ylabel: Y-axis label
            legend: Show legend
        """
        import matplotlib.pyplot as plt
        
        self._setup_style()
        fig, ax = plt.subplots(figsize=self.figsize)
        
        if isinstance(data, dict) and 'x' in data and 'y' in data:
            ax.plot(data['x'], data['y'], marker='o')
            data_points = len(data['x'])
        elif isinstance(data, list):
            data_points = 0
            for series in data:
                label = series.get('label', '')
                ax.plot(series['x'], series['y'], marker='o', label=label)
                data_points += len(series['x'])
            if legend:
                ax.legend()
        else:
            return ChartResult(success=False, error="Invalid data format for line chart")
        
        ax.set_title(title)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        
        plt.tight_layout()
        plt.savefig(output_path, dpi=self.dpi, bbox_inches='tight')
        plt.close()
        
        return ChartResult(
            success=True,
            output_path=output_path,
            chart_type="line",
            title=title,
            data_points=data_points,
        )
    
    def pie(
        self,
        data: Dict[str, Union[int, float]],
        output_path: str,
        title: str = "Pie Chart",
        autopct: str = '%1.1f%%',
        explode: Optional[List[float]] = None,
    ) -> ChartResult:
        """
        Create a pie chart.
        
        Args:
            data: Dictionary of labels to values
            output_path: Path to save chart
            title: Chart title
            autopct: Format string for percentages
            explode: List of explode values for each slice
        """
        import matplotlib.pyplot as plt
        
        self._setup_style()
        fig, ax = plt.subplots(figsize=self.figsize)
        
        labels = list(data.keys())
        values = list(data.values())
        
        ax.pie(values, labels=labels, autopct=autopct, explode=explode)
        ax.set_title(title)
        
        plt.tight_layout()
        plt.savefig(output_path, dpi=self.dpi, bbox_inches='tight')
        plt.close()
        
        return ChartResult(
            success=True,
            output_path=output_path,
            chart_type="pie",
            title=title,
            data_points=len(data),
        )
    
    def scatter(
        self,
        x: List[Union[int, float]],
        y: List[Union[int, float]],
        output_path: str,
        title: str = "Scatter Plot",
        xlabel: str = "",
        ylabel: str = "",
        color: str = "#3498db",
        size: int = 50,
    ) -> ChartResult:
        """
        Create a scatter plot.
        
        Args:
            x: X values
            y: Y values
            output_path: Path to save chart
            title: Chart title
            xlabel: X-axis label
            ylabel: Y-axis label
            color: Point color
            size: Point size
        """
        import matplotlib.pyplot as plt
        
        self._setup_style()
        fig, ax = plt.subplots(figsize=self.figsize)
        
        ax.scatter(x, y, c=color, s=size, alpha=0.7)
        ax.set_title(title)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        
        plt.tight_layout()
        plt.savefig(output_path, dpi=self.dpi, bbox_inches='tight')
        plt.close()
        
        return ChartResult(
            success=True,
            output_path=output_path,
            chart_type="scatter",
            title=title,
            data_points=len(x),
        )
    
    def histogram(
        self,
        data: List[Union[int, float]],
        output_path: str,
        title: str = "Histogram",
        xlabel: str = "",
        ylabel: str = "Frequency",
        bins: int = 20,
        color: str = "#3498db",
    ) -> ChartResult:
        """
        Create a histogram.
        
        Args:
            data: List of values
            output_path: Path to save chart
            title: Chart title
            xlabel: X-axis label
            ylabel: Y-axis label
            bins: Number of bins
            color: Bar color
        """
        import matplotlib.pyplot as plt
        
        self._setup_style()
        fig, ax = plt.subplots(figsize=self.figsize)
        
        ax.hist(data, bins=bins, color=color, edgecolor='white', alpha=0.7)
        ax.set_title(title)
        ax.set_xlabel(xlabel)
        ax.set_ylabel(ylabel)
        
        plt.tight_layout()
        plt.savefig(output_path, dpi=self.dpi, bbox_inches='tight')
        plt.close()
        
        return ChartResult(
            success=True,
            output_path=output_path,
            chart_type="histogram",
            title=title,
            data_points=len(data),
        )
    
    def from_dataframe(
        self,
        df,
        output_path: str,
        chart_type: str = "bar",
        x_col: Optional[str] = None,
        y_col: Optional[str] = None,
        title: str = "",
        **kwargs,
    ) -> ChartResult:
        """
        Create chart from pandas DataFrame.
        
        Args:
            df: Pandas DataFrame
            output_path: Path to save chart
            chart_type: Type of chart
            x_col: Column for x-axis
            y_col: Column for y-axis
            title: Chart title
            **kwargs: Additional chart options
        """
        import matplotlib.pyplot as plt
        
        self._setup_style()
        fig, ax = plt.subplots(figsize=self.figsize)
        
        if chart_type == "bar":
            if x_col and y_col:
                df.plot(kind='bar', x=x_col, y=y_col, ax=ax, **kwargs)
            else:
                df.plot(kind='bar', ax=ax, **kwargs)
        elif chart_type == "line":
            if x_col and y_col:
                df.plot(kind='line', x=x_col, y=y_col, ax=ax, **kwargs)
            else:
                df.plot(kind='line', ax=ax, **kwargs)
        elif chart_type == "pie":
            if y_col:
                df[y_col].plot(kind='pie', ax=ax, autopct='%1.1f%%', **kwargs)
            else:
                df.iloc[:, 0].plot(kind='pie', ax=ax, autopct='%1.1f%%', **kwargs)
        elif chart_type == "scatter":
            if x_col and y_col:
                df.plot(kind='scatter', x=x_col, y=y_col, ax=ax, **kwargs)
        elif chart_type == "histogram":
            if y_col:
                df[y_col].plot(kind='hist', ax=ax, **kwargs)
            else:
                df.iloc[:, 0].plot(kind='hist', ax=ax, **kwargs)
        
        if title:
            ax.set_title(title)
        
        plt.tight_layout()
        plt.savefig(output_path, dpi=self.dpi, bbox_inches='tight')
        plt.close()
        
        return ChartResult(
            success=True,
            output_path=output_path,
            chart_type=chart_type,
            title=title,
            data_points=len(df),
        )
    
    def from_csv(
        self,
        csv_path: str,
        output_path: str,
        chart_type: str = "bar",
        x_col: Optional[str] = None,
        y_col: Optional[str] = None,
        title: str = "",
        **kwargs,
    ) -> ChartResult:
        """
        Create chart from CSV file.
        
        Args:
            csv_path: Path to CSV file
            output_path: Path to save chart
            chart_type: Type of chart
            x_col: Column for x-axis
            y_col: Column for y-axis
            title: Chart title
        """
        import pandas as pd
        
        if not os.path.exists(csv_path):
            return ChartResult(success=False, error=f"CSV not found: {csv_path}")
        
        df = pd.read_csv(csv_path)
        return self.from_dataframe(df, output_path, chart_type, x_col, y_col, title, **kwargs)


# Convenience functions
def chart_bar(data: Dict, output_path: str, title: str = "Bar Chart") -> str:
    """Create bar chart and return path."""
    tool = ChartTool()
    result = tool.bar(data, output_path, title)
    return result.output_path if result.success else ""


def chart_line(data: Dict, output_path: str, title: str = "Line Chart") -> str:
    """Create line chart and return path."""
    tool = ChartTool()
    result = tool.line(data, output_path, title)
    return result.output_path if result.success else ""


def chart_pie(data: Dict, output_path: str, title: str = "Pie Chart") -> str:
    """Create pie chart and return path."""
    tool = ChartTool()
    result = tool.pie(data, output_path, title)
    return result.output_path if result.success else ""


__all__ = [
    "ChartTool",
    "ChartResult",
    "chart_bar",
    "chart_line",
    "chart_pie",
]



"""Spotify Tool for PraisonAI Agents.

Search and manage Spotify music.

Usage:
    from praisonai_tools import SpotifyTool
    
    spotify = SpotifyTool()
    tracks = spotify.search("Beatles")

Environment Variables:
    SPOTIFY_CLIENT_ID: Spotify API client ID
    SPOTIFY_CLIENT_SECRET: Spotify API client secret
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class SpotifyTool(BaseTool):
    """Tool for Spotify music search."""
    
    name = "spotify"
    description = "Search tracks, artists, and playlists on Spotify."
    
    def __init__(
        self,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
    ):
        self.client_id = client_id or os.getenv("SPOTIFY_CLIENT_ID")
        self.client_secret = client_secret or os.getenv("SPOTIFY_CLIENT_SECRET")
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                import spotipy
                from spotipy.oauth2 import SpotifyClientCredentials
            except ImportError:
                raise ImportError("spotipy not installed. Install with: pip install spotipy")
            
            if not self.client_id or not self.client_secret:
                raise ValueError("SPOTIFY_CLIENT_ID and SPOTIFY_CLIENT_SECRET required")
            
            self._client = spotipy.Spotify(
                auth_manager=SpotifyClientCredentials(
                    client_id=self.client_id,
                    client_secret=self.client_secret,
                )
            )
        return self._client
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        track_id: Optional[str] = None,
        artist_id: Optional[str] = None,
        limit: int = 10,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "search":
            return self.search(query=query, limit=limit)
        elif action == "search_artists":
            return self.search_artists(query=query, limit=limit)
        elif action == "get_track":
            return self.get_track(track_id=track_id)
        elif action == "get_artist":
            return self.get_artist(artist_id=artist_id)
        elif action == "get_artist_top_tracks":
            return self.get_artist_top_tracks(artist_id=artist_id)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def search(self, query: str, limit: int = 10) -> List[Dict[str, Any]]:
        """Search tracks."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            results = self.client.search(q=query, type="track", limit=limit)
            tracks = []
            for track in results.get("tracks", {}).get("items", []):
                tracks.append({
                    "id": track["id"],
                    "name": track["name"],
                    "artist": track["artists"][0]["name"] if track["artists"] else None,
                    "album": track["album"]["name"],
                    "duration_ms": track["duration_ms"],
                    "url": track["external_urls"].get("spotify"),
                })
            return tracks
        except Exception as e:
            logger.error(f"Spotify search error: {e}")
            return [{"error": str(e)}]
    
    def search_artists(self, query: str, limit: int = 10) -> List[Dict[str, Any]]:
        """Search artists."""
        if not query:
            return [{"error": "query is required"}]
        
        try:
            results = self.client.search(q=query, type="artist", limit=limit)
            artists = []
            for artist in results.get("artists", {}).get("items", []):
                artists.append({
                    "id": artist["id"],
                    "name": artist["name"],
                    "genres": artist.get("genres", []),
                    "followers": artist.get("followers", {}).get("total"),
                    "url": artist["external_urls"].get("spotify"),
                })
            return artists
        except Exception as e:
            logger.error(f"Spotify search_artists error: {e}")
            return [{"error": str(e)}]
    
    def get_track(self, track_id: str) -> Dict[str, Any]:
        """Get track details."""
        if not track_id:
            return {"error": "track_id is required"}
        
        try:
            track = self.client.track(track_id)
            return {
                "id": track["id"],
                "name": track["name"],
                "artist": track["artists"][0]["name"] if track["artists"] else None,
                "album": track["album"]["name"],
                "duration_ms": track["duration_ms"],
                "popularity": track["popularity"],
                "url": track["external_urls"].get("spotify"),
            }
        except Exception as e:
            logger.error(f"Spotify get_track error: {e}")
            return {"error": str(e)}
    
    def get_artist(self, artist_id: str) -> Dict[str, Any]:
        """Get artist details."""
        if not artist_id:
            return {"error": "artist_id is required"}
        
        try:
            artist = self.client.artist(artist_id)
            return {
                "id": artist["id"],
                "name": artist["name"],
                "genres": artist.get("genres", []),
                "followers": artist.get("followers", {}).get("total"),
                "popularity": artist["popularity"],
                "url": artist["external_urls"].get("spotify"),
            }
        except Exception as e:
            logger.error(f"Spotify get_artist error: {e}")
            return {"error": str(e)}
    
    def get_artist_top_tracks(self, artist_id: str, country: str = "US") -> List[Dict[str, Any]]:
        """Get artist's top tracks."""
        if not artist_id:
            return [{"error": "artist_id is required"}]
        
        try:
            results = self.client.artist_top_tracks(artist_id, country=country)
            tracks = []
            for track in results.get("tracks", []):
                tracks.append({
                    "id": track["id"],
                    "name": track["name"],
                    "album": track["album"]["name"],
                    "popularity": track["popularity"],
                })
            return tracks
        except Exception as e:
            logger.error(f"Spotify get_artist_top_tracks error: {e}")
            return [{"error": str(e)}]


def spotify_search(query: str, limit: int = 10) -> List[Dict[str, Any]]:
    """Search Spotify tracks."""
    return SpotifyTool().search(query=query, limit=limit)



"""Docker Tool for PraisonAI Agents.

Manage Docker containers, images, and execute commands.

Usage:
    from praisonai_tools import DockerTool
    
    docker = DockerTool()
    
    # List containers
    containers = docker.list_containers()
    
    # Run a container
    result = docker.run_container("python:3.11", command="python --version")
    
    # Execute command in container
    output = docker.exec_command(container_id="abc123", command="ls -la")

Environment Variables:
    DOCKER_HOST: Docker daemon URL (optional, uses default socket)
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class DockerTool(BaseTool):
    """Tool for managing Docker containers and images."""
    
    name = "docker"
    description = "Manage Docker containers - list, run, stop, and execute commands."
    
    def __init__(
        self,
        base_url: Optional[str] = None,
    ):
        """Initialize DockerTool.
        
        Args:
            base_url: Docker daemon URL (default: unix socket)
        """
        self.base_url = base_url or os.getenv("DOCKER_HOST")
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        """Lazy-load Docker client."""
        if self._client is None:
            try:
                import docker
            except ImportError:
                raise ImportError("docker not installed. Install with: pip install docker")
            
            if self.base_url:
                self._client = docker.DockerClient(base_url=self.base_url)
            else:
                self._client = docker.from_env()
        return self._client
    
    def run(
        self,
        action: str = "list_containers",
        image: Optional[str] = None,
        container_id: Optional[str] = None,
        command: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        """Execute Docker action."""
        action = action.lower().replace("-", "_")
        
        if action == "list_containers":
            return self.list_containers(**kwargs)
        elif action == "list_images":
            return self.list_images()
        elif action == "run_container":
            return self.run_container(image=image, command=command, **kwargs)
        elif action == "stop_container":
            return self.stop_container(container_id=container_id)
        elif action == "remove_container":
            return self.remove_container(container_id=container_id)
        elif action == "exec_command":
            return self.exec_command(container_id=container_id, command=command)
        elif action == "logs":
            return self.get_logs(container_id=container_id, **kwargs)
        elif action == "pull_image":
            return self.pull_image(image=image)
        elif action == "info":
            return self.get_info()
        else:
            return {"error": f"Unknown action: {action}"}
    
    def list_containers(self, all_containers: bool = False) -> List[Dict[str, Any]]:
        """List Docker containers.
        
        Args:
            all_containers: Include stopped containers
            
        Returns:
            List of container info
        """
        try:
            containers = self.client.containers.list(all=all_containers)
            
            return [
                {
                    "id": c.short_id,
                    "name": c.name,
                    "image": c.image.tags[0] if c.image.tags else c.image.short_id,
                    "status": c.status,
                    "created": str(c.attrs.get("Created", "")),
                }
                for c in containers
            ]
        except Exception as e:
            logger.error(f"Docker list_containers error: {e}")
            return [{"error": str(e)}]
    
    def list_images(self) -> List[Dict[str, Any]]:
        """List Docker images.
        
        Returns:
            List of image info
        """
        try:
            images = self.client.images.list()
            
            return [
                {
                    "id": img.short_id,
                    "tags": img.tags,
                    "size": f"{img.attrs.get('Size', 0) / 1024 / 1024:.1f} MB",
                    "created": str(img.attrs.get("Created", "")),
                }
                for img in images
            ]
        except Exception as e:
            logger.error(f"Docker list_images error: {e}")
            return [{"error": str(e)}]
    
    def run_container(
        self,
        image: str,
        command: Optional[str] = None,
        name: Optional[str] = None,
        detach: bool = True,
        remove: bool = False,
        environment: Optional[Dict[str, str]] = None,
        ports: Optional[Dict[str, int]] = None,
        volumes: Optional[Dict[str, Dict]] = None,
    ) -> Dict[str, Any]:
        """Run a Docker container.
        
        Args:
            image: Image name/tag
            command: Command to run
            name: Container name
            detach: Run in background
            remove: Remove after exit
            environment: Environment variables
            ports: Port mappings {container_port: host_port}
            volumes: Volume mappings
            
        Returns:
            Container info or output
        """
        if not image:
            return {"error": "image is required"}
        
        try:
            kwargs = {
                "image": image,
                "detach": detach,
                "remove": remove,
            }
            
            if command:
                kwargs["command"] = command
            if name:
                kwargs["name"] = name
            if environment:
                kwargs["environment"] = environment
            if ports:
                kwargs["ports"] = ports
            if volumes:
                kwargs["volumes"] = volumes
            
            container = self.client.containers.run(**kwargs)
            
            if detach:
                return {
                    "success": True,
                    "container_id": container.short_id,
                    "name": container.name,
                    "status": container.status,
                }
            else:
                # For non-detached, container is the output bytes
                return {
                    "success": True,
                    "output": container.decode("utf-8") if isinstance(container, bytes) else str(container),
                }
        except Exception as e:
            logger.error(f"Docker run_container error: {e}")
            return {"error": str(e)}
    
    def stop_container(self, container_id: str, timeout: int = 10) -> Dict[str, Any]:
        """Stop a running container.
        
        Args:
            container_id: Container ID or name
            timeout: Seconds to wait before killing
            
        Returns:
            Result
        """
        if not container_id:
            return {"error": "container_id is required"}
        
        try:
            container = self.client.containers.get(container_id)
            container.stop(timeout=timeout)
            return {"success": True, "container_id": container.short_id, "status": "stopped"}
        except Exception as e:
            logger.error(f"Docker stop_container error: {e}")
            return {"error": str(e)}
    
    def remove_container(self, container_id: str, force: bool = False) -> Dict[str, Any]:
        """Remove a container.
        
        Args:
            container_id: Container ID or name
            force: Force removal of running container
            
        Returns:
            Result
        """
        if not container_id:
            return {"error": "container_id is required"}
        
        try:
            container = self.client.containers.get(container_id)
            container.remove(force=force)
            return {"success": True, "container_id": container_id, "status": "removed"}
        except Exception as e:
            logger.error(f"Docker remove_container error: {e}")
            return {"error": str(e)}
    
    def exec_command(self, container_id: str, command: str) -> Dict[str, Any]:
        """Execute command in a running container.
        
        Args:
            container_id: Container ID or name
            command: Command to execute
            
        Returns:
            Command output
        """
        if not container_id:
            return {"error": "container_id is required"}
        if not command:
            return {"error": "command is required"}
        
        try:
            container = self.client.containers.get(container_id)
            exit_code, output = container.exec_run(command)
            
            return {
                "success": exit_code == 0,
                "exit_code": exit_code,
                "output": output.decode("utf-8") if output else "",
            }
        except Exception as e:
            logger.error(f"Docker exec_command error: {e}")
            return {"error": str(e)}
    
    def get_logs(self, container_id: str, tail: int = 100) -> Dict[str, Any]:
        """Get container logs.
        
        Args:
            container_id: Container ID or name
            tail: Number of lines from end
            
        Returns:
            Log output
        """
        if not container_id:
            return {"error": "container_id is required"}
        
        try:
            container = self.client.containers.get(container_id)
            logs = container.logs(tail=tail)
            
            return {
                "container_id": container.short_id,
                "logs": logs.decode("utf-8") if logs else "",
            }
        except Exception as e:
            logger.error(f"Docker get_logs error: {e}")
            return {"error": str(e)}
    
    def pull_image(self, image: str) -> Dict[str, Any]:
        """Pull a Docker image.
        
        Args:
            image: Image name/tag
            
        Returns:
            Result
        """
        if not image:
            return {"error": "image is required"}
        
        try:
            img = self.client.images.pull(image)
            return {
                "success": True,
                "image_id": img.short_id,
                "tags": img.tags,
            }
        except Exception as e:
            logger.error(f"Docker pull_image error: {e}")
            return {"error": str(e)}
    
    def get_info(self) -> Dict[str, Any]:
        """Get Docker system info.
        
        Returns:
            System info
        """
        try:
            info = self.client.info()
            return {
                "containers": info.get("Containers"),
                "containers_running": info.get("ContainersRunning"),
                "images": info.get("Images"),
                "docker_version": info.get("ServerVersion"),
                "os": info.get("OperatingSystem"),
                "architecture": info.get("Architecture"),
                "memory": f"{info.get('MemTotal', 0) / 1024 / 1024 / 1024:.1f} GB",
                "cpus": info.get("NCPU"),
            }
        except Exception as e:
            logger.error(f"Docker get_info error: {e}")
            return {"error": str(e)}


def list_docker_containers(all_containers: bool = False) -> List[Dict[str, Any]]:
    """List Docker containers."""
    return DockerTool().list_containers(all_containers=all_containers)


def run_docker_container(image: str, command: Optional[str] = None) -> Dict[str, Any]:
    """Run a Docker container."""
    return DockerTool().run_container(image=image, command=command)



"""
Repo Tool - Git repository operations.

Provides:
- Getting repository info
- Listing commits (log)
- Getting diffs
- Listing files
- Extracting changelog information
"""

import logging
from dataclasses import dataclass, field
from pathlib import Path
from typing import Dict, List, Optional, Union

import sys
import os
_dir = os.path.dirname(os.path.abspath(__file__))
if _dir not in sys.path:
    sys.path.insert(0, _dir)

try:
    from .base import RecipeToolBase
except ImportError:
    from base import RecipeToolBase

logger = logging.getLogger(__name__)


@dataclass
class CommitInfo:
    """Information about a git commit."""
    hash: str
    short_hash: str
    author: str
    author_email: str
    date: str
    message: str
    files_changed: int = 0
    insertions: int = 0
    deletions: int = 0


@dataclass
class RepoInfo:
    """Information about a git repository."""
    path: str
    name: str
    current_branch: str
    remote_url: Optional[str] = None
    is_dirty: bool = False
    commit_count: int = 0
    last_commit: Optional[CommitInfo] = None
    tags: List[str] = field(default_factory=list)
    branches: List[str] = field(default_factory=list)
    
    def to_dict(self) -> Dict:
        """Convert to dictionary."""
        return {
            "path": self.path,
            "name": self.name,
            "current_branch": self.current_branch,
            "remote_url": self.remote_url,
            "is_dirty": self.is_dirty,
            "commit_count": self.commit_count,
            "tags": self.tags,
            "branches": self.branches,
        }


class RepoTool(RecipeToolBase):
    """
    Git repository operations tool.
    
    Provides repository info, commit history, diffs, and file listing.
    """
    
    name = "repo_tool"
    description = "Git repository operations"
    
    def check_dependencies(self) -> Dict[str, bool]:
        """Check for git."""
        return {
            "git": self._check_binary("git"),
        }
    
    def _git(
        self,
        args: List[str],
        repo_path: Optional[Path] = None,
    ) -> str:
        """Run a git command and return output."""
        self.require_dependencies(["git"])
        
        cmd = ["git"]
        if repo_path:
            cmd.extend(["-C", str(repo_path)])
        cmd.extend(args)
        
        result = self._run_command(cmd)
        if result.returncode != 0:
            raise RuntimeError(f"git command failed: {result.stderr}")
        
        return result.stdout.strip()
    
    def info(self, path: Union[str, Path]) -> RepoInfo:
        """
        Get information about a git repository.
        
        Args:
            path: Path to repository
            
        Returns:
            RepoInfo with repository details
        """
        path = Path(path).resolve()
        
        if not (path / ".git").exists():
            raise ValueError(f"Not a git repository: {path}")
        
        # Get basic info
        name = path.name
        
        # Current branch
        try:
            current_branch = self._git(["rev-parse", "--abbrev-ref", "HEAD"], path)
        except RuntimeError:
            current_branch = "unknown"
        
        # Remote URL
        try:
            remote_url = self._git(["remote", "get-url", "origin"], path)
        except RuntimeError:
            remote_url = None
        
        # Is dirty?
        try:
            status = self._git(["status", "--porcelain"], path)
            is_dirty = len(status) > 0
        except RuntimeError:
            is_dirty = False
        
        # Commit count
        try:
            count_str = self._git(["rev-list", "--count", "HEAD"], path)
            commit_count = int(count_str)
        except (RuntimeError, ValueError):
            commit_count = 0
        
        # Tags
        try:
            tags_str = self._git(["tag", "-l"], path)
            tags = tags_str.split("\n") if tags_str else []
        except RuntimeError:
            tags = []
        
        # Branches
        try:
            branches_str = self._git(["branch", "-a", "--format=%(refname:short)"], path)
            branches = branches_str.split("\n") if branches_str else []
        except RuntimeError:
            branches = []
        
        # Last commit
        try:
            commits = self.log(path, limit=1)
            last_commit = commits[0] if commits else None
        except RuntimeError:
            last_commit = None
        
        return RepoInfo(
            path=str(path),
            name=name,
            current_branch=current_branch,
            remote_url=remote_url,
            is_dirty=is_dirty,
            commit_count=commit_count,
            last_commit=last_commit,
            tags=tags,
            branches=branches,
        )
    
    def log(
        self,
        path: Union[str, Path],
        limit: int = 50,
        since: Optional[str] = None,
        until: Optional[str] = None,
        author: Optional[str] = None,
        grep: Optional[str] = None,
    ) -> List[CommitInfo]:
        """
        Get commit history.
        
        Args:
            path: Path to repository
            limit: Maximum number of commits
            since: Start date/ref (e.g., "2024-01-01" or "v1.0.0")
            until: End date/ref
            author: Filter by author
            grep: Filter by commit message
            
        Returns:
            List of CommitInfo objects
        """
        path = Path(path).resolve()
        
        # Format: hash|short_hash|author|email|date|message
        format_str = "%H|%h|%an|%ae|%aI|%s"
        
        args = [
            "log",
            f"--format={format_str}",
            f"-n{limit}",
        ]
        
        if since:
            args.append(f"--since={since}")
        if until:
            args.append(f"--until={until}")
        if author:
            args.append(f"--author={author}")
        if grep:
            args.append(f"--grep={grep}")
        
        output = self._git(args, path)
        
        if not output:
            return []
        
        commits = []
        for line in output.split("\n"):
            if not line:
                continue
            
            parts = line.split("|", 5)
            if len(parts) < 6:
                continue
            
            commits.append(CommitInfo(
                hash=parts[0],
                short_hash=parts[1],
                author=parts[2],
                author_email=parts[3],
                date=parts[4],
                message=parts[5],
            ))
        
        return commits
    
    def diff(
        self,
        path: Union[str, Path],
        ref1: str = "HEAD~1",
        ref2: str = "HEAD",
        name_only: bool = False,
        stat: bool = False,
    ) -> str:
        """
        Get diff between two refs.
        
        Args:
            path: Path to repository
            ref1: First ref (default: HEAD~1)
            ref2: Second ref (default: HEAD)
            name_only: Only show file names
            stat: Show diffstat
            
        Returns:
            Diff output as string
        """
        path = Path(path).resolve()
        
        args = ["diff"]
        
        if name_only:
            args.append("--name-only")
        elif stat:
            args.append("--stat")
        
        args.extend([ref1, ref2])
        
        return self._git(args, path)
    
    def files(
        self,
        path: Union[str, Path],
        ref: str = "HEAD",
        pattern: Optional[str] = None,
    ) -> List[str]:
        """
        List files in repository.
        
        Args:
            path: Path to repository
            ref: Git ref (default: HEAD)
            pattern: Glob pattern to filter files
            
        Returns:
            List of file paths
        """
        path = Path(path).resolve()
        
        args = ["ls-tree", "-r", "--name-only", ref]
        
        output = self._git(args, path)
        
        if not output:
            return []
        
        files = output.split("\n")
        
        if pattern:
            import fnmatch
            files = [f for f in files if fnmatch.fnmatch(f, pattern)]
        
        return files
    
    def get_file_content(
        self,
        path: Union[str, Path],
        file_path: str,
        ref: str = "HEAD",
    ) -> str:
        """
        Get content of a file at a specific ref.
        
        Args:
            path: Path to repository
            file_path: Path to file within repo
            ref: Git ref (default: HEAD)
            
        Returns:
            File content as string
        """
        path = Path(path).resolve()
        
        return self._git(["show", f"{ref}:{file_path}"], path)
    
    def get_changelog_commits(
        self,
        path: Union[str, Path],
        since_tag: Optional[str] = None,
    ) -> List[CommitInfo]:
        """
        Get commits for changelog generation.
        
        Args:
            path: Path to repository
            since_tag: Start from this tag (default: last tag)
            
        Returns:
            List of commits since tag
        """
        path = Path(path).resolve()
        
        # Find last tag if not specified
        if since_tag is None:
            try:
                since_tag = self._git(["describe", "--tags", "--abbrev=0", "HEAD^"], path)
            except RuntimeError:
                # No previous tag, get all commits
                return self.log(path, limit=100)
        
        # Get commits since tag
        return self.log(path, limit=500, since=since_tag)
    
    def categorize_commits(
        self,
        commits: List[CommitInfo],
    ) -> Dict[str, List[CommitInfo]]:
        """
        Categorize commits by type (feat, fix, docs, etc.).
        
        Args:
            commits: List of commits to categorize
            
        Returns:
            Dict mapping category to commits
        """
        categories = {
            "feat": [],
            "fix": [],
            "docs": [],
            "style": [],
            "refactor": [],
            "perf": [],
            "test": [],
            "chore": [],
            "other": [],
        }
        
        for commit in commits:
            msg = commit.message.lower()
            
            # Check for conventional commit prefixes
            categorized = False
            for cat in categories.keys():
                if msg.startswith(f"{cat}:") or msg.startswith(f"{cat}("):
                    categories[cat].append(commit)
                    categorized = True
                    break
            
            if not categorized:
                # Heuristic categorization
                if any(kw in msg for kw in ["add", "new", "feature", "implement"]):
                    categories["feat"].append(commit)
                elif any(kw in msg for kw in ["fix", "bug", "patch", "resolve"]):
                    categories["fix"].append(commit)
                elif any(kw in msg for kw in ["doc", "readme", "comment"]):
                    categories["docs"].append(commit)
                elif any(kw in msg for kw in ["test", "spec"]):
                    categories["test"].append(commit)
                else:
                    categories["other"].append(commit)
        
        # Remove empty categories
        return {k: v for k, v in categories.items() if v}


# Convenience functions
def repo_info(path: Union[str, Path], verbose: bool = False) -> RepoInfo:
    """Get repository info."""
    return RepoTool(verbose=verbose).info(path)


def repo_log(
    path: Union[str, Path],
    limit: int = 50,
    since: Optional[str] = None,
    verbose: bool = False,
) -> List[CommitInfo]:
    """Get commit history."""
    return RepoTool(verbose=verbose).log(path, limit, since)


def repo_diff(
    path: Union[str, Path],
    ref1: str = "HEAD~1",
    ref2: str = "HEAD",
    verbose: bool = False,
) -> str:
    """Get diff between refs."""
    return RepoTool(verbose=verbose).diff(path, ref1, ref2)


def repo_files(
    path: Union[str, Path],
    pattern: Optional[str] = None,
    verbose: bool = False,
) -> List[str]:
    """List files in repository."""
    return RepoTool(verbose=verbose).files(path, pattern=pattern)



"""Tests for base observability classes."""

import pytest
from datetime import datetime
from unittest.mock import MagicMock, patch

from praisonai_tools.observability.base import (
    BaseObservabilityProvider,
    Span,
    SpanContext,
    SpanKind,
    SpanStatus,
    Trace,
)
from praisonai_tools.observability.config import ObservabilityConfig


class TestSpanContext:
    """Tests for SpanContext."""
    
    def test_new_without_parent(self):
        """Test creating new context without parent."""
        ctx = SpanContext.new()
        assert ctx.trace_id is not None
        assert ctx.span_id is not None
        assert ctx.parent_span_id is None
    
    def test_new_with_parent(self):
        """Test creating new context with parent."""
        parent = SpanContext.new()
        child = SpanContext.new(parent)
        assert child.trace_id == parent.trace_id
        assert child.span_id != parent.span_id
        assert child.parent_span_id == parent.span_id


class TestSpan:
    """Tests for Span."""
    
    def test_span_creation(self):
        """Test span creation."""
        ctx = SpanContext.new()
        span = Span(name="test", kind=SpanKind.LLM, context=ctx)
        assert span.name == "test"
        assert span.kind == SpanKind.LLM
        assert span.status == SpanStatus.UNSET
        assert span.start_time is not None
    
    def test_span_end(self):
        """Test ending a span."""
        ctx = SpanContext.new()
        span = Span(name="test", kind=SpanKind.TOOL, context=ctx)
        span.end(SpanStatus.OK)
        assert span.end_time is not None
        assert span.status == SpanStatus.OK
    
    def test_span_set_error(self):
        """Test setting error on span."""
        ctx = SpanContext.new()
        span = Span(name="test", kind=SpanKind.AGENT, context=ctx)
        span.set_error(ValueError("test error"))
        assert span.status == SpanStatus.ERROR
        assert span.error_message == "test error"
        assert span.error_type == "ValueError"
    
    def test_span_add_event(self):
        """Test adding event to span."""
        ctx = SpanContext.new()
        span = Span(name="test", kind=SpanKind.CUSTOM, context=ctx)
        span.add_event("test_event", {"key": "value"})
        assert len(span.events) == 1
        assert span.events[0]["name"] == "test_event"
    
    def test_span_duration(self):
        """Test span duration calculation."""
        ctx = SpanContext.new()
        span = Span(name="test", kind=SpanKind.LLM, context=ctx)
        assert span.duration_ms is None
        span.end()
        assert span.duration_ms is not None
        assert span.duration_ms >= 0
    
    def test_span_to_dict(self):
        """Test span serialization."""
        ctx = SpanContext.new()
        span = Span(
            name="test",
            kind=SpanKind.LLM,
            context=ctx,
            model="gpt-4",
            input_tokens=100,
            output_tokens=50,
        )
        span.end()
        data = span.to_dict()
        assert data["name"] == "test"
        assert data["kind"] == "llm"
        assert data["model"] == "gpt-4"
        assert data["input_tokens"] == 100
        assert data["output_tokens"] == 50


class TestTrace:
    """Tests for Trace."""
    
    def test_trace_creation(self):
        """Test trace creation."""
        trace = Trace(trace_id="test-123", name="test-trace")
        assert trace.trace_id == "test-123"
        assert trace.name == "test-trace"
        assert len(trace.spans) == 0
    
    def test_trace_add_span(self):
        """Test adding span to trace."""
        trace = Trace(trace_id="test-123", name="test-trace")
        ctx = SpanContext(trace_id="test-123", span_id="span-1")
        span = Span(name="test-span", kind=SpanKind.LLM, context=ctx)
        trace.add_span(span)
        assert len(trace.spans) == 1
    
    def test_trace_end(self):
        """Test ending a trace."""
        trace = Trace(trace_id="test-123", name="test-trace")
        trace.end()
        assert trace.end_time is not None
    
    def test_trace_to_dict(self):
        """Test trace serialization."""
        trace = Trace(
            trace_id="test-123",
            name="test-trace",
            session_id="session-1",
            user_id="user-1",
        )
        trace.end()
        data = trace.to_dict()
        assert data["trace_id"] == "test-123"
        assert data["name"] == "test-trace"
        assert data["session_id"] == "session-1"


class TestObservabilityConfig:
    """Tests for ObservabilityConfig."""
    
    def test_default_config(self):
        """Test default configuration."""
        config = ObservabilityConfig()
        assert config.enabled is True
        assert config.trace_llm_calls is True
        assert config.trace_tool_calls is True
        assert config.batch_size == 100
    
    def test_config_from_env(self):
        """Test config from environment variables."""
        with patch.dict("os.environ", {
            "PRAISONAI_OBS_PROVIDER": "langfuse",
            "PRAISONAI_OBS_PROJECT": "test-project",
            "PRAISONAI_OBS_ENABLED": "false",
        }):
            config = ObservabilityConfig.from_env()
            assert config.provider == "langfuse"
            assert config.project_name == "test-project"
            assert config.enabled is False
    
    def test_config_merge(self):
        """Test config merge."""
        config = ObservabilityConfig(provider="langfuse")
        merged = config.merge(project_name="new-project", session_id="sess-1")
        assert merged.provider == "langfuse"
        assert merged.project_name == "new-project"
        assert merged.session_id == "sess-1"


class ConcreteProvider(BaseObservabilityProvider):
    """Concrete implementation for testing."""
    
    name = "test"
    
    def init(self, **kwargs):
        self._initialized = True
        return True
    
    def shutdown(self):
        self._initialized = False
    
    def is_available(self):
        return True
    
    def check_connection(self):
        return True, "OK"
    
    def export_span(self, span):
        return True
    
    def export_trace(self, trace):
        return True


class TestBaseObservabilityProvider:
    """Tests for BaseObservabilityProvider."""
    
    def test_provider_init(self):
        """Test provider initialization."""
        provider = ConcreteProvider()
        assert provider.init() is True
        assert provider._initialized is True
    
    def test_provider_start_trace(self):
        """Test starting a trace."""
        provider = ConcreteProvider()
        provider.init()
        trace = provider.start_trace("test-trace", session_id="sess-1")
        assert trace is not None
        assert trace.name == "test-trace"
        assert trace.session_id == "sess-1"
    
    def test_provider_start_span(self):
        """Test starting a span."""
        provider = ConcreteProvider()
        provider.init()
        provider.start_trace("test-trace")
        span = provider.start_span("test-span", SpanKind.LLM)
        assert span is not None
        assert span.name == "test-span"
        assert span.kind == SpanKind.LLM
    
    def test_provider_trace_context_manager(self):
        """Test trace context manager."""
        provider = ConcreteProvider()
        provider.init()
        with provider.trace("test-trace") as trace:
            assert trace is not None
            assert trace.name == "test-trace"
    
    def test_provider_span_context_manager(self):
        """Test span context manager."""
        provider = ConcreteProvider()
        provider.init()
        with provider.span("test-span", SpanKind.TOOL) as span:
            assert span is not None
            assert span.name == "test-span"
    
    def test_provider_log_llm_call(self):
        """Test logging LLM call."""
        provider = ConcreteProvider()
        provider.init()
        provider.start_trace("test")
        span = provider.log_llm_call(
            model="gpt-4",
            input_messages="Hello",
            output="Hi there",
            input_tokens=10,
            output_tokens=5,
        )
        assert span is not None
        assert span.model == "gpt-4"
        assert span.input_tokens == 10
    
    def test_provider_log_tool_call(self):
        """Test logging tool call."""
        provider = ConcreteProvider()
        provider.init()
        provider.start_trace("test")
        span = provider.log_tool_call(
            tool_name="search",
            tool_input={"query": "test"},
            tool_output={"results": []},
        )
        assert span is not None
        assert span.tool_name == "search"
    
    def test_provider_decorator(self):
        """Test function decorator."""
        provider = ConcreteProvider()
        provider.init()
        provider.start_trace("test")
        
        @provider.decorator("decorated-func", SpanKind.CUSTOM)
        def my_func():
            return "result"
        
        result = my_func()
        assert result == "result"



"""Google Drive Tool for PraisonAI Agents.

Google Drive file operations.

Usage:
    from praisonai_tools import GoogleDriveTool
    
    drive = GoogleDriveTool()
    files = drive.list_files()

Environment Variables:
    GOOGLE_DRIVE_CREDENTIALS_FILE: Path to credentials.json
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class GoogleDriveTool(BaseTool):
    """Tool for Google Drive operations."""
    
    name = "google_drive"
    description = "Google Drive file operations."
    
    def __init__(self, credentials_file: Optional[str] = None):
        self.credentials_file = credentials_file or os.getenv("GOOGLE_DRIVE_CREDENTIALS_FILE", "credentials.json")
        self._service = None
        super().__init__()
    
    @property
    def service(self):
        if self._service is None:
            try:
                from google.oauth2.credentials import Credentials
                from google_auth_oauthlib.flow import InstalledAppFlow
                from google.auth.transport.requests import Request
                from googleapiclient.discovery import build
            except ImportError:
                raise ImportError("google-api-python-client required")
            
            SCOPES = ["https://www.googleapis.com/auth/drive"]
            creds = None
            token_file = "drive_token.json"
            
            if os.path.exists(token_file):
                creds = Credentials.from_authorized_user_file(token_file, SCOPES)
            
            if not creds or not creds.valid:
                if creds and creds.expired and creds.refresh_token:
                    creds.refresh(Request())
                else:
                    flow = InstalledAppFlow.from_client_secrets_file(self.credentials_file, SCOPES)
                    creds = flow.run_local_server(port=0)
                with open(token_file, "w") as token:
                    token.write(creds.to_json())
            
            self._service = build("drive", "v3", credentials=creds)
        return self._service
    
    def run(
        self,
        action: str = "list",
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "list":
            return self.list_files(**kwargs)
        elif action == "get":
            return self.get_file(file_id=kwargs.get("file_id"))
        elif action == "search":
            return self.search(query=kwargs.get("query"))
        elif action == "download":
            return self.download(file_id=kwargs.get("file_id"), output_path=kwargs.get("output_path"))
        else:
            return {"error": f"Unknown action: {action}"}
    
    def list_files(self, page_size: int = 20) -> List[Dict[str, Any]]:
        """List files."""
        try:
            results = self.service.files().list(
                pageSize=page_size,
                fields="files(id, name, mimeType, modifiedTime)"
            ).execute()
            return results.get("files", [])
        except Exception as e:
            logger.error(f"Google Drive list error: {e}")
            return [{"error": str(e)}]
    
    def get_file(self, file_id: str) -> Dict[str, Any]:
        """Get file metadata."""
        if not file_id:
            return {"error": "file_id is required"}
        try:
            return self.service.files().get(fileId=file_id).execute()
        except Exception as e:
            logger.error(f"Google Drive get error: {e}")
            return {"error": str(e)}
    
    def search(self, query: str) -> List[Dict[str, Any]]:
        """Search files."""
        if not query:
            return [{"error": "query is required"}]
        try:
            results = self.service.files().list(
                q=f"name contains '{query}'",
                fields="files(id, name, mimeType)"
            ).execute()
            return results.get("files", [])
        except Exception as e:
            logger.error(f"Google Drive search error: {e}")
            return [{"error": str(e)}]
    
    def download(self, file_id: str, output_path: str) -> Dict[str, Any]:
        """Download file."""
        if not file_id or not output_path:
            return {"error": "file_id and output_path are required"}
        try:
            from googleapiclient.http import MediaIoBaseDownload
            import io
            
            request = self.service.files().get_media(fileId=file_id)
            fh = io.BytesIO()
            downloader = MediaIoBaseDownload(fh, request)
            done = False
            while not done:
                _, done = downloader.next_chunk()
            
            with open(output_path, "wb") as f:
                f.write(fh.getvalue())
            return {"success": True, "output_path": output_path}
        except Exception as e:
            logger.error(f"Google Drive download error: {e}")
            return {"error": str(e)}


def list_drive_files(page_size: int = 20) -> List[Dict[str, Any]]:
    """List Google Drive files."""
    return GoogleDriveTool().list_files(page_size=page_size)



"""Weather Tool for PraisonAI Agents.

Get current weather, forecasts, and air quality data.

Usage:
    from praisonai_tools import WeatherTool
    
    weather = WeatherTool()  # Uses OPENWEATHER_API_KEY env var
    
    # Get current weather
    result = weather.get_current("London")
    
    # Get forecast
    forecast = weather.get_forecast("New York", days=5)

Environment Variables:
    OPENWEATHER_API_KEY: OpenWeatherMap API key (free tier available)
"""

import os
import logging
from typing import Any, Dict, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class WeatherTool(BaseTool):
    """Tool for getting weather information."""
    
    name = "weather"
    description = "Get current weather, forecasts, and air quality for any location."
    
    def __init__(
        self,
        api_key: Optional[str] = None,
        units: str = "metric",
    ):
        """Initialize WeatherTool.
        
        Args:
            api_key: OpenWeatherMap API key (or use OPENWEATHER_API_KEY env var)
            units: Temperature units - "metric" (Celsius), "imperial" (Fahrenheit), "standard" (Kelvin)
        """
        self.api_key = api_key or os.getenv("OPENWEATHER_API_KEY")
        self.units = units
        self.base_url = "https://api.openweathermap.org/data/2.5"
        self.geo_url = "https://api.openweathermap.org/geo/1.0"
        super().__init__()
    
    def _request(self, url: str, params: Dict) -> Dict:
        """Make API request."""
        try:
            import requests
        except ImportError:
            return {"error": "requests not installed. Install with: pip install requests"}
        
        try:
            params["appid"] = self.api_key
            response = requests.get(url, params=params, timeout=10)
            response.raise_for_status()
            return response.json()
        except Exception as e:
            logger.error(f"Weather API error: {e}")
            return {"error": str(e)}
    
    def _geocode(self, location: str) -> Optional[Dict]:
        """Convert location name to coordinates."""
        if not self.api_key:
            return None
        url = f"{self.geo_url}/direct"
        result = self._request(url, {"q": location, "limit": 1})
        if isinstance(result, list) and result:
            return {"lat": result[0]["lat"], "lon": result[0]["lon"], "name": result[0].get("name", location)}
        return None
    
    def run(
        self,
        action: str = "current",
        location: Optional[str] = None,
        lat: Optional[float] = None,
        lon: Optional[float] = None,
        days: int = 5,
        **kwargs
    ) -> Union[str, Dict[str, Any]]:
        """Execute weather action.
        
        Args:
            action: "current", "forecast", "air_quality"
            location: City name (e.g., "London", "New York, US")
            lat: Latitude (alternative to location)
            lon: Longitude (alternative to location)
            days: Forecast days (for forecast action)
        """
        if not self.api_key:
            return {"error": "OPENWEATHER_API_KEY not configured"}
        
        action = action.lower()
        if action == "current":
            return self.get_current(location=location, lat=lat, lon=lon)
        elif action == "forecast":
            return self.get_forecast(location=location, lat=lat, lon=lon, days=days)
        elif action in ("air", "air_quality", "pollution"):
            return self.get_air_quality(location=location, lat=lat, lon=lon)
        else:
            return {"error": f"Unknown action: {action}. Use 'current', 'forecast', or 'air_quality'."}
    
    def get_current(
        self,
        location: Optional[str] = None,
        lat: Optional[float] = None,
        lon: Optional[float] = None,
    ) -> Dict[str, Any]:
        """Get current weather for a location.
        
        Args:
            location: City name
            lat: Latitude
            lon: Longitude
            
        Returns:
            Current weather data
        """
        if not self.api_key:
            return {"error": "OPENWEATHER_API_KEY not configured"}
        
        # Get coordinates
        if lat is None or lon is None:
            if not location:
                return {"error": "Location or coordinates required"}
            geo = self._geocode(location)
            if not geo:
                return {"error": f"Could not find location: {location}"}
            lat, lon = geo["lat"], geo["lon"]
            location = geo["name"]
        
        url = f"{self.base_url}/weather"
        params = {"lat": lat, "lon": lon, "units": self.units}
        result = self._request(url, params)
        
        if "error" in result:
            return result
        
        # Format response
        temp_unit = "°C" if self.units == "metric" else "°F" if self.units == "imperial" else "K"
        speed_unit = "m/s" if self.units == "metric" else "mph" if self.units == "imperial" else "m/s"
        
        return {
            "location": result.get("name", location),
            "country": result.get("sys", {}).get("country", ""),
            "temperature": f"{result['main']['temp']}{temp_unit}",
            "feels_like": f"{result['main']['feels_like']}{temp_unit}",
            "humidity": f"{result['main']['humidity']}%",
            "pressure": f"{result['main']['pressure']} hPa",
            "wind_speed": f"{result['wind']['speed']} {speed_unit}",
            "description": result["weather"][0]["description"] if result.get("weather") else "",
            "icon": result["weather"][0]["icon"] if result.get("weather") else "",
            "visibility": f"{result.get('visibility', 0) / 1000} km",
            "clouds": f"{result.get('clouds', {}).get('all', 0)}%",
        }
    
    def get_forecast(
        self,
        location: Optional[str] = None,
        lat: Optional[float] = None,
        lon: Optional[float] = None,
        days: int = 5,
    ) -> Dict[str, Any]:
        """Get weather forecast.
        
        Args:
            location: City name
            lat: Latitude
            lon: Longitude
            days: Number of days (max 5 for free tier)
            
        Returns:
            Forecast data
        """
        if not self.api_key:
            return {"error": "OPENWEATHER_API_KEY not configured"}
        
        if lat is None or lon is None:
            if not location:
                return {"error": "Location or coordinates required"}
            geo = self._geocode(location)
            if not geo:
                return {"error": f"Could not find location: {location}"}
            lat, lon = geo["lat"], geo["lon"]
            location = geo["name"]
        
        url = f"{self.base_url}/forecast"
        params = {"lat": lat, "lon": lon, "units": self.units, "cnt": min(days * 8, 40)}
        result = self._request(url, params)
        
        if "error" in result:
            return result
        
        temp_unit = "°C" if self.units == "metric" else "°F" if self.units == "imperial" else "K"
        
        forecasts = []
        for item in result.get("list", []):
            forecasts.append({
                "datetime": item["dt_txt"],
                "temperature": f"{item['main']['temp']}{temp_unit}",
                "feels_like": f"{item['main']['feels_like']}{temp_unit}",
                "humidity": f"{item['main']['humidity']}%",
                "description": item["weather"][0]["description"] if item.get("weather") else "",
                "wind_speed": item["wind"]["speed"],
                "rain_chance": item.get("pop", 0) * 100,
            })
        
        return {
            "location": result.get("city", {}).get("name", location),
            "country": result.get("city", {}).get("country", ""),
            "forecasts": forecasts,
        }
    
    def get_air_quality(
        self,
        location: Optional[str] = None,
        lat: Optional[float] = None,
        lon: Optional[float] = None,
    ) -> Dict[str, Any]:
        """Get air quality/pollution data.
        
        Args:
            location: City name
            lat: Latitude
            lon: Longitude
            
        Returns:
            Air quality data
        """
        if not self.api_key:
            return {"error": "OPENWEATHER_API_KEY not configured"}
        
        if lat is None or lon is None:
            if not location:
                return {"error": "Location or coordinates required"}
            geo = self._geocode(location)
            if not geo:
                return {"error": f"Could not find location: {location}"}
            lat, lon = geo["lat"], geo["lon"]
            location = geo["name"]
        
        url = f"{self.base_url}/air_pollution"
        params = {"lat": lat, "lon": lon}
        result = self._request(url, params)
        
        if "error" in result:
            return result
        
        if not result.get("list"):
            return {"error": "No air quality data available"}
        
        data = result["list"][0]
        aqi = data["main"]["aqi"]
        aqi_labels = {1: "Good", 2: "Fair", 3: "Moderate", 4: "Poor", 5: "Very Poor"}
        
        return {
            "location": location,
            "aqi": aqi,
            "aqi_label": aqi_labels.get(aqi, "Unknown"),
            "components": {
                "co": f"{data['components'].get('co', 0)} μg/m³",
                "no2": f"{data['components'].get('no2', 0)} μg/m³",
                "o3": f"{data['components'].get('o3', 0)} μg/m³",
                "pm2_5": f"{data['components'].get('pm2_5', 0)} μg/m³",
                "pm10": f"{data['components'].get('pm10', 0)} μg/m³",
            },
        }


def get_weather(location: str, units: str = "metric") -> Dict[str, Any]:
    """Get current weather for a location."""
    return WeatherTool(units=units).get_current(location=location)


def get_forecast(location: str, days: int = 5, units: str = "metric") -> Dict[str, Any]:
    """Get weather forecast for a location."""
    return WeatherTool(units=units).get_forecast(location=location, days=days)


def get_air_quality(location: str) -> Dict[str, Any]:
    """Get air quality for a location."""
    return WeatherTool().get_air_quality(location=location)



"""AWS Lambda Tool for PraisonAI Agents.

Invoke AWS Lambda functions.

Usage:
    from praisonai_tools import AWSLambdaTool
    
    lambda_tool = AWSLambdaTool()
    result = lambda_tool.invoke("my-function", {"key": "value"})

Environment Variables:
    AWS_ACCESS_KEY_ID: AWS access key
    AWS_SECRET_ACCESS_KEY: AWS secret key
    AWS_REGION: AWS region
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class AWSLambdaTool(BaseTool):
    """Tool for AWS Lambda."""
    
    name = "aws_lambda"
    description = "Invoke AWS Lambda functions."
    
    def __init__(self, region: Optional[str] = None):
        self.region = region or os.getenv("AWS_REGION", "us-east-1")
        self._client = None
        super().__init__()
    
    @property
    def client(self):
        if self._client is None:
            try:
                import boto3
            except ImportError:
                raise ImportError("boto3 not installed")
            self._client = boto3.client("lambda", region_name=self.region)
        return self._client
    
    def run(
        self,
        action: str = "invoke",
        function_name: Optional[str] = None,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "invoke":
            return self.invoke(function_name=function_name, payload=kwargs.get("payload"))
        elif action == "list_functions":
            return self.list_functions()
        else:
            return {"error": f"Unknown action: {action}"}
    
    def invoke(self, function_name: str, payload: Dict = None) -> Dict[str, Any]:
        """Invoke Lambda function."""
        if not function_name:
            return {"error": "function_name is required"}
        
        try:
            import json
            resp = self.client.invoke(
                FunctionName=function_name,
                InvocationType="RequestResponse",
                Payload=json.dumps(payload or {}),
            )
            result = json.loads(resp["Payload"].read())
            return {"status_code": resp["StatusCode"], "result": result}
        except Exception as e:
            logger.error(f"Lambda invoke error: {e}")
            return {"error": str(e)}
    
    def list_functions(self) -> List[Dict[str, Any]]:
        """List Lambda functions."""
        try:
            resp = self.client.list_functions()
            return [
                {"name": f["FunctionName"], "runtime": f.get("Runtime"), "memory": f.get("MemorySize")}
                for f in resp.get("Functions", [])
            ]
        except Exception as e:
            logger.error(f"Lambda list_functions error: {e}")
            return [{"error": str(e)}]


def invoke_lambda(function_name: str, payload: Dict = None) -> Dict[str, Any]:
    """Invoke Lambda function."""
    return AWSLambdaTool().invoke(function_name=function_name, payload=payload)



"""Brave Search Tool for PraisonAI Agents.

Web search using Brave Search API.

Usage:
    from praisonai_tools import BraveSearchTool
    
    brave = BraveSearchTool()
    results = brave.search("AI news")

Environment Variables:
    BRAVE_API_KEY: Brave Search API key
"""

import os
import logging
from typing import Any, Dict, List, Optional, Union

from praisonai_tools.tools.base import BaseTool

logger = logging.getLogger(__name__)


class BraveSearchTool(BaseTool):
    """Tool for Brave Search."""
    
    name = "brave_search"
    description = "Search the web using Brave Search API."
    
    def __init__(self, api_key: Optional[str] = None):
        self.api_key = api_key or os.getenv("BRAVE_API_KEY")
        super().__init__()
    
    def run(
        self,
        action: str = "search",
        query: Optional[str] = None,
        count: int = 10,
        **kwargs
    ) -> Union[str, Dict[str, Any], List[Dict[str, Any]]]:
        action = action.lower().replace("-", "_")
        
        if action == "search":
            return self.search(query=query, count=count)
        elif action == "news":
            return self.news(query=query, count=count)
        else:
            return {"error": f"Unknown action: {action}"}
    
    def search(self, query: str, count: int = 10) -> List[Dict[str, Any]]:
        """Search the web."""
        if not query:
            return [{"error": "query is required"}]
        if not self.api_key:
            return [{"error": "BRAVE_API_KEY required"}]
        
        try:
            import requests
        except ImportError:
            return [{"error": "requests not installed"}]
        
        try:
            headers = {"X-Subscription-Token": self.api_key}
            params = {"q": query, "count": count}
            resp = requests.get(
                "https://api.search.brave.com/res/v1/web/search",
                headers=headers,
                params=params,
                timeout=10,
            )
            data = resp.json()
            
            results = []
            for item in data.get("web", {}).get("results", [])[:count]:
                results.append({
                    "title": item.get("title"),
                    "url": item.get("url"),
                    "description": item.get("description"),
                })
            return results
        except Exception as e:
            logger.error(f"Brave search error: {e}")
            return [{"error": str(e)}]
    
    def news(self, query: str, count: int = 10) -> List[Dict[str, Any]]:
        """Search news."""
        if not query:
            return [{"error": "query is required"}]
        if not self.api_key:
            return [{"error": "BRAVE_API_KEY required"}]
        
        try:
            import requests
        except ImportError:
            return [{"error": "requests not installed"}]
        
        try:
            headers = {"X-Subscription-Token": self.api_key}
            params = {"q": query, "count": count}
            resp = requests.get(
                "https://api.search.brave.com/res/v1/news/search",
                headers=headers,
                params=params,
                timeout=10,
            )
            data = resp.json()
            
            results = []
            for item in data.get("results", [])[:count]:
                results.append({
                    "title": item.get("title"),
                    "url": item.get("url"),
                    "description": item.get("description"),
                    "age": item.get("age"),
                })
            return results
        except Exception as e:
            logger.error(f"Brave news error: {e}")
            return [{"error": str(e)}]


def brave_search(query: str, count: int = 10) -> List[Dict[str, Any]]:
    """Search with Brave."""
    return BraveSearchTool().search(query=query, count=count)



"""Tests for DataTool."""

import json
import pytest
from pathlib import Path

from praisonai_tools.recipe_tools.data_tool import DataTool, data_profile, data_clean, data_convert


class TestDataTool:
    """Unit tests for DataTool."""
    
    @pytest.fixture
    def tool(self):
        return DataTool(verbose=True)
    
    @pytest.mark.unit
    def test_check_dependencies(self, tool):
        """Test dependency checking."""
        deps = tool.check_dependencies()
        assert "pandas" in deps
        assert "pyarrow" in deps
    
    @pytest.mark.unit
    def test_detect_format(self, tool):
        """Test format detection."""
        assert tool._detect_format(Path("test.csv")) == "csv"
        assert tool._detect_format(Path("test.json")) == "json"
        assert tool._detect_format(Path("test.parquet")) == "parquet"
        assert tool._detect_format(Path("test.xlsx")) == "excel"
    
    @pytest.mark.unit
    def test_profile_csv(self, tool, sample_csv, temp_dir):
        """Test profiling a CSV file."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        result = tool.profile(sample_csv)
        
        assert result.path == str(sample_csv)
        assert result.format == "csv"
        assert result.row_count == 10
        assert result.column_count == 6
        assert len(result.columns) == 6
        
        # Check column profiles
        id_col = next(c for c in result.columns if c.name == "id")
        assert id_col.dtype == "int64"
        assert id_col.null_count == 0
    
    @pytest.mark.unit
    def test_profile_json(self, tool, sample_json, temp_dir):
        """Test profiling a JSON file."""
        if not sample_json.exists():
            pytest.skip("Sample JSON not found")
        
        result = tool.profile(sample_json)
        
        assert result.format == "json"
        assert result.row_count == 5
    
    @pytest.mark.unit
    def test_clean_csv(self, tool, sample_csv, temp_dir):
        """Test cleaning a CSV file."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        output_path = temp_dir / "cleaned.csv"
        result = tool.clean(sample_csv, output_path, drop_duplicates=True)
        
        assert result.exists()
        assert result.stat().st_size > 0
    
    @pytest.mark.unit
    def test_convert_csv_to_json(self, tool, sample_csv, temp_dir):
        """Test converting CSV to JSON."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        output_path = temp_dir / "output.json"
        result = tool.convert(sample_csv, output_path)
        
        assert result.exists()
        
        # Verify JSON is valid
        with open(result) as f:
            data = json.load(f)
        assert len(data) == 10
    
    @pytest.mark.unit
    def test_infer_schema(self, tool, sample_csv, temp_dir):
        """Test schema inference."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        schema = tool.infer_schema(sample_csv, output_format="json_schema")
        
        assert "$schema" in schema
        assert "properties" in schema
        assert "id" in schema["properties"]
    
    @pytest.mark.unit
    def test_generate_profile_report(self, tool, sample_csv, temp_dir):
        """Test HTML report generation."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        output_path = temp_dir / "report.html"
        result = tool.generate_profile_report(sample_csv, output_path)
        
        assert result.exists()
        content = result.read_text()
        assert "<html>" in content
        assert "Data Profile" in content


class TestDataToolConvenienceFunctions:
    """Test convenience functions."""
    
    @pytest.mark.unit
    def test_data_profile(self, sample_csv):
        """Test data_profile function."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        result = data_profile(sample_csv)
        assert result.row_count == 10
    
    @pytest.mark.unit
    def test_data_clean(self, sample_csv, temp_dir):
        """Test data_clean function."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        output = temp_dir / "cleaned.csv"
        result = data_clean(sample_csv, output)
        assert result.exists()
    
    @pytest.mark.unit
    def test_data_convert(self, sample_csv, temp_dir):
        """Test data_convert function."""
        if not sample_csv.exists():
            pytest.skip("Sample CSV not found")
        
        output = temp_dir / "output.json"
        result = data_convert(sample_csv, output)
        assert result.exists()
